/*
 * @fluidListStruct.java Version 1.1 03/21/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package fluid;

import fluid.fluidStruct;

/** CLASS fluidListStruct
 *  <p> This Class holds the Brine Data List Information.
 *
 *  List composed from,
 *  Common Cations, Anions, Acids, Salts and Hydrate Nomenclature
 *  Missouri University of Science and Technology
 *  http://web.mst.edu/~tbone/subjects/tbone/chem1/Chem%201%20Compound%20Sheet%20FS08.pdf
 *
 *  @version 1.1 03/21/2016
 *  @author  John R. Victorine
 */

public class fluidListStruct
{
  // Fluid Level Data

  public int         iCount   = 0;    // Total Number of Records
  public fluidStruct stItem[] = null; // Array of Brine Data Structures

  // Fluid Level Data Identifiers

  public  int _KEY    = -1; // KEY

  // Other Measured Well Data
  public  int _FLUID  = -1; // Total Fluid
  public  int _CUT    = -1; // Oil Cut (%)
  public  int _BOPD   = -1; // Barrels of Oil per Day
  public  int _LEVEL  = -1; // Fluid Level (ft)
  public  int _JNTS   = -1; // Joints in Well
  public  int _BHP    = -1; // Bottom Hole Pressure
  public  int _CO2    = -1; // CO2 (%)
  public  int _MCFD   = -1; // MCF / D

  /** Method delete()
   * <p> This method will force set the Objects to null and force Java to
   *     free unwanted memory
   */

  public void delete()
  {
	int i=0;

    for (i=0; i<iCount; i++)
    {
      if (stItem[i] != null)
        stItem[i].delete();
      stItem[i] = null;
    }

    iCount = 0;
  }

  /** Method checkData(int iCurve)
   * <p> This method will return a true if the data is present
   * @param iCurve = the requested curve type
   * @return bValue true the data is present and false it is not there
   */

  public boolean checkData(int iCurve)
  {
    boolean bValue = false;

    switch (iCurve)
    {
      // Fluid Level Data Identifiers

      case fluid.fluidStandardTools._FLUID:     // Total Fluid
        if ( _FLUID > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._CUT:     // Oil Cut (%)
        if ( _CUT > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._BOPD:     // Barrels of Oil per Day
        if ( _BOPD > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._LEVEL:     // Fluid Level (ft)
        if ( _LEVEL > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._JNTS:     // Joints in Well
        if ( _JNTS > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._BHP:     // Bottom Hole Pressure (psig)
        if ( _BHP > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._CO2:     // CO2 (%)
        if ( _CO2 > -1) bValue = true;
        break;

      case fluid.fluidStandardTools._MCFD:   // MCF/D
        if ( _MCFD > -1) bValue = true;
        break;
    }

    return (bValue);
  }

  /** Method setID(int iCurve)
   * <p> This method will set the anion & cation Identifier
   * @param  iCurve = the requested curve type
   */

  public void setID(int iCurve)
  {
	if (iCount > 0)
	{
	  for (int i=0; i<iCount; i++)
	  {
        switch (iCurve)
        {
          // Fluid Level Data Identifiers

          case fluid.fluidStandardTools._FLUID:     // Total Fluid
            if (stItem[i].dTotal > 0.0) _FLUID = 1;
            break;

          case fluid.fluidStandardTools._CUT:     // Oil Cut (%)
            if (stItem[i].dOilCut > 0.0) _CUT = 1;
            break;

          case fluid.fluidStandardTools._BOPD:     // Barrels of Oil per Day
            if (stItem[i].dBOPD > 0.0) _BOPD = 1;
            break;

          case fluid.fluidStandardTools._LEVEL:     // Fluid Level (ft)
            if (stItem[i].dLevel > 0.0) _LEVEL = 1;
            break;

          case fluid.fluidStandardTools._JNTS:     // Joints in Well
            if (stItem[i].dJoints > 0.0) _JNTS = 1;
            break;

          case fluid.fluidStandardTools._BHP:     // Bottom Hole Pressure (psig)
            if (stItem[i].dBHP > 0.0) _BHP = 1;
            break;

          case fluid.fluidStandardTools._CO2:     // CO2 (%)
            if (stItem[i].dCO2 > 0.0) _CO2 = 1;
            break;

          case fluid.fluidStandardTools._MCFD:   // MCF/D
            if (stItem[i].dMCFD > 0.0) _MCFD = 1;
            break;
		}
  	  }
    }
  }

  /** Method getData(int iCurve)
   * <p> This method will retrive the curve data requested
   * @param  iCurve = the requested curve type
   * @return data   = the data requested.
   */

  public double[] getData(int iCurve)
  {
    double data[] = null;

    data = new double[iCount];

    for (int i=0; i<iCount; i++)
    {
      switch (iCurve)
      {
        case fluid.fluidStandardTools._FLUID:     // Total Fluid
          data[i] = stItem[i].dTotal;
          break;

        case fluid.fluidStandardTools._CUT:     // Oil Cut (%)
          data[i] = stItem[i].dOilCut;
          break;

        case fluid.fluidStandardTools._BOPD:     // Barrels of Oil per Day
          data[i] = stItem[i].dBOPD;
          break;

        case fluid.fluidStandardTools._LEVEL:     // Fluid Level (ft)
          data[i] = stItem[i].dLevel;
          break;

        case fluid.fluidStandardTools._JNTS:     // Joints in Well
          data[i] = stItem[i].dJoints;
          break;

        case fluid.fluidStandardTools._BHP:     // Bottom Hole Pressure (psig)
          data[i] = stItem[i].dBHP;
          break;

        case fluid.fluidStandardTools._CO2:     // CO2 (%)
          data[i] = stItem[i].dCO2;
          break;

        case fluid.fluidStandardTools._MCFD:   // MCF/D
          data[i] = stItem[i].dMCFD;
          break;
      }
    }

    return (data);
  }

  /** Method getData(int iCurve, int iRow)
   * <p> This method will retrive the curve data requested
   * @param  iCurve = the requested curve type
   * @param  iRow   = the row
   * @return data   = the data requested.
   */

  public double getData( int iCurve, int iRow )
  {
    double data = 0.0;

    switch (iCurve)
    {
      case fluid.fluidStandardTools._FLUID:     // Total Fluid
        data = stItem[iRow].dTotal;
        break;

      case fluid.fluidStandardTools._CUT:     // Oil Cut (%)
        data = stItem[iRow].dOilCut;
        break;

      case fluid.fluidStandardTools._BOPD:     // Barrels of Oil per Day
        data = stItem[iRow].dBOPD;
        break;

      case fluid.fluidStandardTools._LEVEL:     // Fluid Level (ft)
        data = stItem[iRow].dLevel;
        break;

      case fluid.fluidStandardTools._JNTS:     // Joints in Well
        data = stItem[iRow].dJoints;
        break;

      case fluid.fluidStandardTools._BHP:     // Bottom Hole Pressure (psig)
        data = stItem[iRow].dBHP;
        break;

      case fluid.fluidStandardTools._CO2:     // CO2 (%)
        data = stItem[iRow].dCO2;
        break;

      case fluid.fluidStandardTools._MCFD:   // MCF/D
        data = stItem[iRow].dMCFD;
        break;
    }

    if (data < 0.0)
      data = 0.0;

    return (data);
  }

  /** Method setData(int iCurve, int iRow, double data)
   * <p> This method will retrive the curve data requested
   * @param  iCurve = the requested curve type
   * @param  iRow   = the row
   * @return data   = the data requested.
   */

  public void setData( int iCurve, int iRow, double data )
  {
    switch (iCurve)
    {
      case fluid.fluidStandardTools._FLUID:     // Total Fluid
        stItem[iRow].dTotal = data;
        break;

      case fluid.fluidStandardTools._CUT:     // Oil Cut (%)
        stItem[iRow].dOilCut = data;
        break;

      case fluid.fluidStandardTools._BOPD:     // Barrels of Oil per Day
        stItem[iRow].dBOPD = data;
        break;

      case fluid.fluidStandardTools._LEVEL:     // Fluid Level (ft)
        stItem[iRow].dLevel = data;
        break;

      case fluid.fluidStandardTools._JNTS:     // Joints in Well
        stItem[iRow].dJoints = data;
        break;

      case fluid.fluidStandardTools._BHP:     // Bottom Hole Pressure (psig)
        stItem[iRow].dBHP = data;
        break;

      case fluid.fluidStandardTools._CO2:     // CO2 (%)
        stItem[iRow].dCO2 = data;
        break;

      case fluid.fluidStandardTools._MCFD:   // MCF/D
        stItem[iRow].dMCFD = data;
        break;
    }
  }

  /** Method setData(int iCurve, int iRow, String sData)
   * <p> This method will retrive the curve data requested
   * @param  iCurve = the requested curve type
   * @param  iRow   = the row
   * @return sData  = the data requested.
   */

  public void setData( int iCurve, int iRow, String sData )
  {
    switch (iCurve)
    {
        // Brine Data Identifiers

        case fluid.fluidStandardTools._NAME: // Well Name
          stItem[iRow].sName = new String(sData);
          break;
        case fluid.fluidStandardTools._API: // API-Number
          stItem[iRow].sAPI = new String(sData);
          break;
        case fluid.fluidStandardTools._LOC: // Location
          stItem[iRow].sLocation = new String(sData);
          break;
        case fluid.fluidStandardTools._FIELD: // Field
          stItem[iRow].sField = new String(sData);
          break;
        case fluid.fluidStandardTools._COUNTY: // County
          stItem[iRow].sCounty = new String(sData);
          break;
        case fluid.fluidStandardTools._STATE: // State
          stItem[iRow].state = new String(sData);
          break;
        case fluid.fluidStandardTools._SRC: // Source
          stItem[iRow].source = new String(sData);
          break;
        case fluid.fluidStandardTools._DATE: // Date
          stItem[iRow].sRecovery = new String(sData);
          break;
	}
  }
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
