/*
 * @fluidUtility.java Version 1.1 03/21/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package fluid;

import fluid.fluidStruct;
import fluid.fluidListStruct;

import iqstrat.iqstratHeadersStruct;

/** Class fluidUtility
 *  <p> This Class will provide basic utilities for the Rock data structures.
 *
 *  @version 1.1 03/21/2016
 *  @author  John R. Victorine
 */

public class fluidUtility
{
  /** Method copyList()
   * <p> This method will copy one Brine Data list structure to another
   * @param  stOld = Old Brine Data list data structure
   * @return stNew = New Brine Data list data structure
   */

  public static fluidListStruct copyList(fluidListStruct stOld)
  {
    int i,j;
    fluidListStruct stNew = null;

    if (stOld != null)
    {
      if (stOld.iCount > 0)
      {
        stNew        = new fluidListStruct();
        stNew.iCount = stOld.iCount;
        stNew.stItem = new fluidStruct[stOld.iCount];

        // Fluid Level Data Identifiers

        stNew._KEY    = stOld._KEY;   // KEY

        stNew._FLUID  = stOld._FLUID; // Total Fluid
        stNew._CUT    = stOld._CUT;   // Oil Cut (%)
        stNew._BOPD   = stOld._BOPD;  // Barrels of Oil per Day
        stNew._LEVEL  = stOld._LEVEL; // Fluid Level (ft)
        stNew._JNTS   = stOld._JNTS;  // Joints in Well
        stNew._BHP    = stOld._BHP;   // Bottom Hole Pressure
        stNew._CO2    = stOld._CO2;   // CO2 (%)
        stNew._MCFD   = stOld._MCFD;  // MCF / D

        for (i=0; i<stOld.iCount; i++)
        {
          stNew.stItem[i] = copy(stOld.stItem[i]);
        }
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static fluidListStruct transfer(fluidListStruct stOld)
  {
    fluidListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method copyFluidHeader()
   * <p> This method will copy one Fluid Level Header Data list structure to another
   * @param  stOld = Old Brine Data list data structure
   * @return stNew = New Brine Data list data structure
   */

  public static fluidListStruct copyFluidHeader(fluidListStruct stOld)
  {
    int i,j;
    fluidListStruct stNew = null;

    if (stOld != null)
    {
      stNew._KEY    = stOld._KEY;   // KEY

      stNew._FLUID  = stOld._FLUID; // Total Fluid
      stNew._CUT    = stOld._CUT;   // Oil Cut (%)
      stNew._BOPD   = stOld._BOPD;  // Barrels of Oil per Day
      stNew._LEVEL  = stOld._LEVEL; // Fluid Level (ft)
      stNew._JNTS   = stOld._JNTS;  // Joints in Well
      stNew._BHP    = stOld._BHP;   // Bottom Hole Pressure
      stNew._CO2    = stOld._CO2;   // CO2 (%)
      stNew._MCFD   = stOld._MCFD;  // MCF / D
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the Brine Data to an existing list
   * @param  stBrine = The Brine Data Structure
   * @param  st     = The Old Brine Data List Data Structure
   * @return st     = The new Brine Data List Data Structure.
   */

  public static fluidListStruct add( fluidStruct stBrine,
                                     fluidListStruct st )
  {
    int             i=0;
    int             iRecords = 0;
    int             iCount   = 0;
    fluidListStruct stTemp   = null;

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp        = copyFluidHeader( st );
    stTemp.stItem = new fluidStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new fluidStruct();
    stTemp.stItem[iCount] = copy(stBrine);
    iCount++;

    stTemp.iCount = iCount;

    st        = copyFluidHeader( st );
    st.stItem = new fluidStruct[iRecords];
    st.iCount = iCount;

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method modify()
   * <p> This method will modify a data structrure in an existing list
   * @param  sKEY    = The Primary Key of the data to be removed
   * @param  stBlock = The fossil structure that will be replaced
   * @param  st      = The List Data Structure
   * @return st      = The new List Data Structure.
   */

  public static fluidListStruct modify( String sKEY,
                                        fluidStruct stBlock,
                                        fluidListStruct st)
  {
    int i=0;
    fluidListStruct stTemp = null;

    if (st != null)
    {
      stTemp        = copyFluidHeader( st );
      stTemp.stItem = new fluidStruct[st.iCount];
      stTemp.iCount = st.iCount;

      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            stTemp.stItem[i] = copy(stBlock);
          }
          else
          {
            stTemp.stItem[i] = copy(st.stItem[i]);
          }
        }
      }

      st.delete();

      st        = copyFluidHeader( stTemp );
      st.stItem = new fluidStruct[stTemp.iCount];
      st.iCount = stTemp.iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method remove()
   * <p> This method will remove a data structure from an existing list
   * @param  sKEY   = The Primary Key of the data to be removed
   * @param  st     = The Old List Data Structure
   * @return st     = The new List Data Structure.
   */

  public static fluidListStruct remove( String sKEY, fluidListStruct st )
  {
    int i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    int    iFound   = -1;
    fluidListStruct stTemp = null;

    if (st.iCount == 1)
    {
      st.delete();
      st = null;
    }
    else
    {
      if (st != null)
      {
        iRecords = st.iCount-1;

        stTemp        = copyFluidHeader( st );
        stTemp.stItem = new fluidStruct[iRecords];

        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            iFound = i;
          }
        }
      }

      if (st != null)
      {
        if (st.iCount > 0)
        {
          for (i=0; i<st.iCount; i++)
          {
            if (iCount < iRecords)
            {
              if (i != iFound)
              {
                stTemp.stItem[iCount] = copy(st.stItem[i]);
                iCount++;
              }
            }
          }
        }

        st.delete();
      }

      stTemp.iCount = iCount;

      st        = copyFluidHeader( stTemp );
      st.stItem = new fluidStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method copy()
   * <p> This method will copy one Brine data structure to another
   * @param  stOld = old Brine data structure
   * @return stNew = new Brine data structure
   */

  public static fluidStruct copy (fluidStruct stOld)
  {
    fluidStruct stNew = new fluidStruct();

    if (stOld != null)
    {
        // Brine Data Identifiers

        stNew.sKID    = new String( stOld.sKID );    // KID
        stNew.sKEY    = new String( stOld.sKEY );    // KEY

        // Identification Information
        stNew.sWellKID   = new String( stOld.sWellKID );    // Well Header KID
        stNew.sAPI       = new String( stOld.sAPI );        // API-Number
        stNew.sName      = new String( stOld.sName );       // Name
        stNew.sField     = new String( stOld.sField );      // Field
        stNew.sCounty    = new String( stOld.sCounty );     // County
        stNew.state      = new String( stOld.state );       // State
        stNew.sLocation  = new String( stOld.sLocation );   // Location (TRS)

        // XY Position Information
        stNew.dLatitude  = stOld.dLatitude;  // Latitude
        stNew.dLongitude = stOld.dLongitude; // Longitude
        stNew.dZone      = stOld.dZone;      // UTM Zone
        stNew.dUTMx      = stOld.dUTMx;      // UTM X-Position
        stNew.dUTMy      = stOld.dUTMy;      // UTM Y-Position

        // Elevation Data
        stNew.dGL        = stOld.dGL;   // Ground Level
        stNew.dKB        = stOld.dKB;   // Kelly Bushing
        stNew.dDF        = stOld.dDF;   // Derrick Floor
        stNew.dTGT       = stOld.dTGT;  // TGT

        // Group
        stNew.group_id   = new String(stOld.group_id);  // Group ID - LAB or FIELD
        stNew.group_no   = stOld.group_no;  // Group Number - Data organized by Date

        // Data
        stNew.source     = new String(stOld.source);     // Core Source
        stNew.sRecovery  = new String(stOld.sRecovery);  // Recovery Date

        // Other Measured Well Data
        stNew.dTotal  = stOld.dTotal;  // Total Fluid
        stNew.dOilCut = stOld.dOilCut; // Oil Cut (%)
        stNew.dBOPD   = stOld.dBOPD;   // Barrels of Oil per Day
        stNew.dLevel  = stOld.dLevel;  // Fluid Level (ft)
        stNew.dJoints = stOld.dJoints; // Joints in Well
        stNew.dBHP    = stOld.dBHP;    // Estimated Bottom Hole Pressure
        stNew.dCO2    = stOld.dCO2;    // CO2 (%)
        stNew.dMCFD   = stOld.dMCFD;   // MCF/D
    }

    return (stNew);
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  st = fluid data list structure
   * @param  By = By Identifier
   * @return st = sorted fluid data list structure
   */

  public static fluidListStruct bubbleSort(fluidListStruct st, int By)
  {
    boolean swappedOnPrevRun  = true;
    fluidStruct stTemp   = null;
    double      d1       = 0.0;
    double      d2       = 0.0;

    if (st != null)
    {
      if (st.iCount > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<st.iCount-1; i++)
          {
            // if current element is greater than the next swap the two elements

            switch (By)
            {
			  case 0:
			    d1 = st.stItem[i].group_no;
			    d2 = st.stItem[i+1].group_no;
                break;
			  case 1:
			    d1 = st.stItem[i].dLatitude;
			    d2 = st.stItem[i+1].dLatitude;
                break;
		    }

            if(d1 > d2)
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              stTemp = copy(st.stItem[i]);

              // set element i+1 to where i used to be

              st.stItem[i] = copy(st.stItem[i+1]);

              // release the old i from temp into i+1 slot

              st.stItem[i+1] = copy(stTemp);
            }
          }
        }
      }
    }

    return (st);
  }

  /** Method print()
   *  <p> This method will print the contents of the Fluid List Structure
   *  @param st = fluid List structure
   */

  public static void print(fluidListStruct st)
  {
	System.out.println("FLUID DATA");
	if (st != null)
	{
	  for (int i=0; i<st.iCount; i++)
	  {
		 System.out.print(st.stItem[i].sName+" ("+st.stItem[i].sAPI+") ");
         System.out.print("NO:"    + st.stItem[i].group_no  +" "+
                          "CUT: "  + st.stItem[i].dOilCut   +" "+ // Oil Cut (%)
                          "BOPD: " + st.stItem[i].dBOPD     +" "+ // Barrels of Oil per Day
                          "LEVEL: "+ st.stItem[i].dLevel    +" "+ // Fluid Level (ft)
                          "JNTS: " + st.stItem[i].dJoints   +" "+ // Joints in Well
                          "BHP: "  + st.stItem[i].dBHP      +" "+ // Estimated Bottom Hole Pressure
                          "CO2: "  + st.stItem[i].dCO2      +" "+ // CO2 (%)
                          "MCF: "  + st.stItem[i].dMCFD); // MCF/D
         System.out.println("");
	  }

	}
  }
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
