/*
 * @gridDataUtility.java Version 1.1 06/25/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid;

import grid.gridDataStruct;              // Grid Data Structure
import grid.math.gridParametersStruct;   // Grid Parameters Data Structure
import grid.math.gridMath;               // Grid Math Class

/** Class gridDataUtility
 *  <p> This Class holds the possible selections from the KGS DB
 *
 *  @version 1.1 06/25/2015
 *  @author  John R. Victorine
 */

public class gridDataUtility
{
  /** Method loadGriddingParametersData()
   * <p> This method will load the Gridding Parameters Data from the initial
   *     Well Header Informaiton Data and compute the Default Gridding
   *     Parameter Data.
   * @param  st = The Grid Data Structure
   * @return st = The Grid Data Structure.
   */

  public static gridDataStruct loadGriddingParametersData( gridDataStruct st )
  {
    int    i      = 0;
    double dXMin  = 0.0;
    double dYMin  = 0.0;

    if (st.stWellList != null)
    {
      st.iRows = st.stWellList.iCount;

      if (st.stWellList.iCount > 0)
      {
        st.dX     = new double[st.stWellList.iCount];
        st.dY     = new double[st.stWellList.iCount];
        st.dZR    = new double[st.stWellList.iCount];
        st.dZG    = new double[st.stWellList.iCount];
        st.dZB    = new double[st.stWellList.iCount];
        st.sAPI   = new String[st.stWellList.iCount];
        st.sName  = new String[st.stWellList.iCount];
        st.status = new String[st.stWellList.iCount];
        st.iTriangle = new int[st.stWellList.iCount];


        // Find the Minimum UTM-X and UTM-Y Value.

        for (i=0; i<st.stWellList.iCount; i++)
        {
          if (i == 0)
          {
            dXMin = st.stWellList.stItem[i].dUTMx;
            dYMin = st.stWellList.stItem[i].dUTMy;
          }

          if (dXMin > st.stWellList.stItem[i].dUTMx)
            dXMin = st.stWellList.stItem[i].dUTMx;

          if (dYMin > st.stWellList.stItem[i].dUTMy)
            dYMin = st.stWellList.stItem[i].dUTMy;
        }

        // Set the X, Y Position relative to the Minimum UTM-X and UTM-Y.

        for (i=0; i<st.stWellList.iCount; i++)
        {
          st.dX[i]        = 3.3 * (st.stWellList.stItem[i].dUTMx - dXMin);
          st.dY[i]        = 3.3 * (st.stWellList.stItem[i].dUTMy - dYMin);
          st.dZR[i]       = 0.0;
          st.dZG[i]       = 0.0;
          st.dZB[i]       = 0.0;
          st.sAPI[i]      = new String(st.stWellList.stItem[i].sAPI);
          st.sName[i]     = new String(st.stWellList.stItem[i].sName);
          st.status[i]    = new String(st.stWellList.stItem[i].status);
          st.iTriangle[i] = 0;
        }

        // Set the Grid for the Well Data

        gridMath pGridding         = new gridMath(st.iRows, st.dX, st.dY);

        // Retrieve the Gridding Parameters from the gridMath Class

        st.stGrid                  = new gridParametersStruct();

        st.stGrid.dCellSize        = pGridding.getGridCellSpacing();
        st.stGrid.dXMin            = pGridding.getMinimumX();
        st.stGrid.dXMax            = pGridding.getMaximumX();
        st.stGrid.dYMin            = pGridding.getMinimumY();
        st.stGrid.dYMax            = pGridding.getMaximumY();
        st.stGrid.iNeighbors       = pGridding.getNumberOfNearestNeighbors();
        st.stGrid.dMaxDistTo1st    = pGridding.getMaxDistanceToNearestDataPoint();
        st.stGrid.dMaxSearchRadius = pGridding.getMaxSearchRadius();
        st.stGrid.dExponent        = pGridding.getInversDistanceExponent();
        st.stGrid.iXCells          = pGridding.getNumberOfRows();
        st.stGrid.iYCells          = pGridding.getNumberOfColumns();
      }
    }

    return (st);
  }

  /** Method recompute()
   *  <p> This method will recompute the Gridding Rows and Columns when the
   *      grid Text field is altered.
   *  @param  dGridCellSize = The Grid Cell Size.
   *  @param  st = The Grid Data Structure.
   *  @return st = The Grid Data Structure.
   */

  public static gridDataStruct recompute(double dGridCellSize, gridDataStruct st)
  {
    String sTemp = new String("0.0");

    // Set the Grid for the Well Data

    if (st != null)
    {
	  if (st.iRows > 0)
	  {
        gridMath pGridding = new gridMath(st.iRows, st.dX, st.dY);
        pGridding.setGridCellSpacing(dGridCellSize);

        st.stGrid.dCellSize        = dGridCellSize;
        st.stGrid.dXMin            = pGridding.getMinimumX();
        st.stGrid.dXMax            = pGridding.getMaximumX();
        st.stGrid.dYMin            = pGridding.getMinimumY();
        st.stGrid.dYMax            = pGridding.getMaximumY();
        st.stGrid.iXCells          = pGridding.getNumberOfRows();
        st.stGrid.iYCells          = pGridding.getNumberOfColumns();
        st.stGrid.dMaxDistTo1st    = pGridding.getMaxDistanceToNearestDataPoint();
        st.stGrid.dMaxSearchRadius = pGridding.getMaxSearchRadius();
        st.stGrid.dExponent        = pGridding.getInversDistanceExponent();
        st.stGrid.iNeighbors       = pGridding.getNumberOfNearestNeighbors();
	  }
	}

    return (st);
  }

  /** Method copyGrid()
   *  <p> This method will copy the grid parameters data structure.
   *  @param  st     = The Gridding Parameters Data Structure.
   *  @return stGrid = The Gridding Parameters Data Structure.
   */

  public static gridParametersStruct copyGrid( gridParametersStruct st )
  {
	gridParametersStruct stGrid = new gridParametersStruct();

    stGrid.dCellSize        = st.dCellSize;
    stGrid.dXMin            = st.dXMin;
    stGrid.dXMax            = st.dXMax;
    stGrid.dYMin            = st.dYMin;
    stGrid.dYMax            = st.dYMax;
    stGrid.iXCells          = st.iXCells;
    stGrid.iYCells          = st.iYCells;
    stGrid.dMaxDistTo1st    = st.dMaxDistTo1st;
    stGrid.dMaxSearchRadius = st.dMaxSearchRadius;
    stGrid.dExponent        = st.dExponent;
    stGrid.iNeighbors       = st.iNeighbors;

    return (stGrid);
  }

}

/*
 *  @version 1.1 06/25/2015
 *  @author  John Victorine
 */
