/*
 * @gridControlDataPanel.java Version 1.1 02/26/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import grid.gridStruct;
import grid.gridDataStruct;
import grid.plot.gridPlotMappingStruct;
import grid.plot.gridPlotMappingListStruct;
import grid.math.gridMath;                   // Grid Math Class
import grid.math.gridParametersStruct;       // Grid Parameters Structure

import fluid.fluidListStruct;                // Brine List Structure

/** Class gridControlDataPanel
 *  <p> This Class will display the KHAN Control Frame.
 *
 *  @version 1.1 02/26/2016
 *  @author  John R. Victorine
 */

public class gridControlDataPanel extends JPanel implements ActionListener
{
  // Other Measured Well Data
  public static final int _FLUID   = fluid.fluidStandardTools._FLUID; // Total Fluid
  public static final int _CUT     = fluid.fluidStandardTools._CUT;   // Oil Cut (%)
  public static final int _BOPD    = fluid.fluidStandardTools._BOPD;  // Barrels of Oil per Day
  public static final int _LEVEL   = fluid.fluidStandardTools._LEVEL; // Fluid Level (ft)
  public static final int _JNTS    = fluid.fluidStandardTools._JNTS;  // Joints in Well
  public static final int _BHP     = fluid.fluidStandardTools._BHP;   // Bottom Hole Pressure
  public static final int _CO2     = fluid.fluidStandardTools._CO2;   // CO2 (%)
  public static final int _MCFD    = fluid.fluidStandardTools._MCFD;  // MCF / D

  public static final int TOTAL    = fluid.fluidStandardTools.TOTAL;  // TOTAL

  public static final int _RED    = 0;
  public static final int _GREEN  = 1;
  public static final int _BLUE   = 2;

  public static final int NONE   = grid.plot.gridPlotMapping.NONE;   // No Label
  public static final int NUMBER = grid.plot.gridPlotMapping.NUMBER; // Order Number
  public static final int API    = grid.plot.gridPlotMapping.API;    // API Number
  public static final int LEASE  = grid.plot.gridPlotMapping.LEASE;  // Lease Name

  // Input Variables

  private Observable      notifier = null;
  private gridStruct      stStruct = null;
  private gridDataStruct  stData   = null;
  private int             iWidth   = 0;

  private fluidListStruct stBrine  = null;

  // Global Variables

  private int iRed        = -1;
  private int iGreen      = -1;
  private int iBlue       = -1;

  private int iAllowed[]  = { _FLUID, _CUT, _BOPD, _LEVEL, _JNTS, _BHP, _CO2, _MCFD };

  private int iTotal      = 0;
  private int iDataType[] = null;
  private int iColor[]    = null;

  private int iEnd        = 0;

  private gridAvgTable pTable    = null; // Fluid Statistics Table

  private gridParametersStruct stGrid    = null; // Gridding Parameters Structure
  private gridMath             pGridding = null; // Grid Math

  private double dColor[][] = {{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}};
  private String sColor[]   = { "", "", ""};

  // Global Frame Widgets

  private JButton btnClear   = new JButton();

  private JLabel     lbl[]   = null;
  private JCheckBox  cb[][]  = null;

  private static final Color GOOD    = new Color(140,  0,211);
  private static final Color _50     = new Color( 65,105,225);
  private static final Color _20     = new Color( 34,139, 34);
  private static final Color CAUTION = new Color(255,128,  0);
  private static final Color BAD     = new Color(255,  0,  0);

  /** Constructor gridControlDataPanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observables Class
   *  @param parent   = The Parent Frame calling this Panel
   *  @param stStruct = The Gridding & Mapping Module Data Structure
   */

  public gridControlDataPanel( Observable     notifier,
                               gridStruct     stStruct,
                               gridDataStruct stData,
                               int            iWidth )
  {
    try
    {
	  this.notifier = notifier;
	  this.stStruct = stStruct;
      this.stData   = stData;
      this.iWidth   = iWidth;

      stBrine = fluid.fluidUtility.copyList( stData.stFluid );

	  setData();

      jbInit();

      setButtons();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("gridControlDataPanel: " +  ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                     sMessage,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   * @throws Exception
   */

  void jbInit() throws Exception
  {
    JPanel pnlTop    = new JPanel();
    JPanel pnlTop1   = new JPanel();
    JPanel pnlCenter = new JPanel();
    JPanel pnlCurves = new JPanel();
    JPanel pnlList   = new JPanel();

    JLabel lblTop    = new JLabel();
    JScrollPane scroll = new JScrollPane();

    // Build Panel

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    pnlTop1.setLayout(new BorderLayout());

    // -- Instructions

    lblTop.setText("Select check boxes from left R-Red to right B-Blue " +
                   "otherwise the color mix will not plot correctly.");
    lblTop.setForeground(Color.red);
    lblTop.setFont(new java.awt.Font("Dialog", 2, 13));

    // -- Build Clear Buttons

    btnClear.setFont(new java.awt.Font("Dialog", 1, 11));
    btnClear.setText("Clear Selection");
    btnClear.addActionListener(this);

    // -- Build Curves Button Panel

    pnlCurves = buildButtonsPanel();

    // -- Statistics of Data Panel

    pnlList.setLayout(new BorderLayout());
    pnlList.setPreferredSize(new Dimension(10, 90));
//    pnlList.setPreferredSize(new Dimension(10, 150));

    pTable = new gridAvgTable();
    scroll = pTable.getScrollPane();

    // Attach Widgets to the Frame

    this.add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlTop1,   BorderLayout.NORTH);
    pnlTop1.add(lblTop,   BorderLayout.WEST);
    pnlTop1.add(btnClear, BorderLayout.EAST);
    pnlTop.add(pnlCurves, BorderLayout.CENTER);
    pnlTop.add(pnlList,   BorderLayout.SOUTH);
    pnlList.add(scroll,   BorderLayout.CENTER);
  }

  /** Method buildButtonsPanel()
   *  <p> This method will build the buttons Panel
   */

  private JPanel buildButtonsPanel()
  {
	int    iHeight    = 0;
	int    iColorVal  = 0;
	JPanel panel      = new JPanel();
	JPanel pnlOther   = new JPanel();
	JPanel pnlAnions  = new JPanel();
	JPanel pnlCations = new JPanel();

	int    iOther     = 0;
	int    iAnions    = 0;
	int    iCations   = 0;
	int    iMaximum   = 0;

	JPanel pnlROW[]   = null;
	JPanel pnlBTNS[]  = null;
	JLabel lblBTNS[]  = null;

	JPanel pnlROWC[]  = null;
	JPanel pnlLBLC[]  = null;
	JLabel lblDesc[]  = null;
	JLabel lblGood[]  = null;
	JPanel pnlCLR[][] = null;
	JLabel lblCLR[][] = null;

    TitledBorder titledBorderOther = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Other");
    titledBorderOther.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderCations = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Cations");
    titledBorderCations.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    TitledBorder titledBorderAnions = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Anions");
    titledBorderAnions.setTitleFont(new java.awt.Font("Dialog", 1, 12));

    panel.setLayout(new GridLayout());

    pnlOther.setBorder(titledBorderOther);
    pnlAnions.setBorder(titledBorderAnions);
    pnlCations.setBorder(titledBorderCations);

    pnlROWC = new JPanel[3];
    pnlLBLC = new JPanel[3];
    lblDesc = new JLabel[3];
    lblGood = new JLabel[3];

    pnlCLR  = new JPanel[3][3];
    lblCLR  = new JLabel[3][3];

    for (int i=0; i<3; i++)
    {
      pnlROWC[i] = new JPanel();
	  pnlROWC[i].setLayout(new BorderLayout());

      pnlLBLC[i] = new JPanel();
      pnlLBLC[i].setLayout(new GridLayout());

      lblDesc[i] = new JLabel();
      lblDesc[i].setText("Curve");

      lblGood[i] = new JLabel();
      lblGood[i].setFont(new java.awt.Font("Monospaced", 1, 11));
      lblGood[i].setText("  ? ");
      pnlLBLC[i].add(lblGood[i]);

      for (int j=0; j<3; j++)
      {
		pnlCLR[i][j] = new JPanel();
		pnlCLR[i][j].setLayout(new BorderLayout());
	    pnlCLR[i][j].setPreferredSize(new Dimension(25, 10));

        lblCLR[i][j] = new JLabel();
        if (j==0) { lblCLR[i][j].setText("R"); }
        if (j==1) { lblCLR[i][j].setText("G"); }
        if (j==2) { lblCLR[i][j].setText("B"); }

        pnlCLR[i][j].add(lblCLR[i][j], BorderLayout.CENTER);
        pnlLBLC[i].add(pnlCLR[i][j], null);
	  }

	  pnlROWC[i].add(pnlLBLC[i], BorderLayout.WEST);
	  pnlROWC[i].add(lblDesc[i], BorderLayout.CENTER);
    }

    if (iTotal > 0)
    {
//	  iHeight = 15 * iTotal;
//      panel.setPreferredSize(new Dimension(10, iHeight));

	  pnlROW  = new JPanel[iTotal];
      pnlBTNS = new JPanel[iTotal];
      lblBTNS = new JLabel[iTotal];
      lbl = new JLabel[iTotal];
      cb  = new JCheckBox[iTotal][3];

      for (int i=0; i<iTotal; i++)
      {
		pnlROW[i]  = new JPanel();
		pnlROW[i].setLayout(new BorderLayout());

		pnlBTNS[i] = new JPanel();
		pnlBTNS[i].setLayout(new GridLayout());

		lblBTNS[i] = new JLabel();
		lblBTNS[i].setText( fluid.fluidStandardTools.CURVES[iDataType[i]][2] );

		switch (iDataType[i])
		{
		  // Other
          case _FLUID: // Total Fluid
          case _CUT:   // Oil Cut (%)
          case _BOPD:  // Barrels of Oil per Day
          case _LEVEL: // Fluid Level (ft)
          case _JNTS:  // Joints in Well
          case _BHP:   // Bottom Hole Pressure
          case _CO2:   // CO2 (%)
          case _MCFD:  // MCF / D
            iOther++;
            break;
     	}

        if (iColor != null)
          iColorVal = iColor[i];

		lbl[i] = new JLabel();
		lbl[i].setFont(new java.awt.Font("Monospaced", 1, 11));
		switch (iColorVal)
		{
		  case 0:
  		    lbl[i].setForeground(BAD);
		    lbl[i].setText( "  0%" );
		    break;
		  case 1:
  		    lbl[i].setForeground(CAUTION);
		    lbl[i].setText( "  5%" );
		    break;
		  case 2:
  		    lbl[i].setForeground(_20);
		    lbl[i].setText( " 20%" );
		    break;
		  case 3:
  		    lbl[i].setForeground(_50);
		    lbl[i].setText( " 50%" );
		    break;
		  case 4:
  		    lbl[i].setForeground(GOOD);
		    lbl[i].setText( "100%" );
		    break;
		}

		pnlBTNS[i].add(lbl[i], null);

	    for (int j=0; j<3; j++)
	    {
          cb[i][j] = new JCheckBox();
          cb[i][j].addActionListener(this);
          pnlBTNS[i].add(cb[i][j], null);
	    }

	    pnlROW[i].add(pnlBTNS[i], BorderLayout.WEST);
	    pnlROW[i].add(lblBTNS[i], BorderLayout.CENTER);
	  }

	  if (iMaximum < iOther)
	    iMaximum = iOther;

	  if (iMaximum < iCations)
	    iMaximum = iCations;

	  if (iMaximum < iAnions)
	    iMaximum = iAnions;

      pnlOther.setLayout(new GridLayout(iMaximum+1, 1));
      pnlCations.setLayout(new GridLayout(iMaximum+1, 1));
      pnlAnions.setLayout(new GridLayout(iMaximum+1, 1));

      pnlOther.add(pnlROWC[0],   null);
      pnlCations.add(pnlROWC[1], null);
      pnlAnions.add(pnlROWC[2],  null);

      for (int i=0; i<iTotal; i++)
      {
		switch (iDataType[i])
		{
		  // Other
          case _FLUID: // Total Fluid
          case _CUT:   // Oil Cut (%)
          case _BOPD:  // Barrels of Oil per Day
          case _LEVEL: // Fluid Level (ft)
          case _JNTS:  // Joints in Well
          case _BHP:   // Bottom Hole Pressure
          case _CO2:   // CO2 (%)
          case _MCFD:  // MCF / D
            pnlOther.add(pnlROW[i], null);
            break;
     	}
	  }
	}

    iHeight = 30 * iMaximum;
    panel.setPreferredSize(new Dimension(iWidth, iHeight));

    panel.add(pnlOther,   null);
//    panel.add(pnlCations, null);
//    panel.add(pnlAnions,  null);

	return (panel);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will deallocate memory.
   */

  public void close()
  {
	int i,j;

    notifier = null;
    stStruct = null;
    stData   = null;

    if (stBrine != null)
      stBrine.delete();
    stBrine  = null;

    iTotal      = 0;
    iDataType   = null;
    iColor      = null;

    if (pTable != null)
      pTable.close();
    pTable = null;

    if (pGridding != null)
      pGridding.delete();
    pGridding = null;

    if (lbl != null)
    {
	  for (i=0; i<iTotal; i++)
	  {
		lbl[i] = null;
	  }
	}
    lbl = null;

    if (cb != null)
    {
      for (i=0; i<iTotal; i++)
      {
	    for (j=0; j<3; j++)
	    {
          cb[i][j] = null;
	    }
	  }
	}
    cb = null;

    btnClear = null;
  }

  /* ======================================================================= *
   * ---------------------------- GET METHODS ------------------------------ *
   * ======================================================================= */

  /** Method getColor()
   *  <p> This method will count the number anions and cations that can be used
   *  @param  st     = The fluid List Structure
   *  @return iCount = The total number of Cations & Anions
   */

  private static int[] getColor( fluidListStruct st, int iDataType[] )
  {
	int    iColor[] = null;
	int    iCount = 0; // Add API-Number to List
	double data[] = null;
    int    iMin   = 0;
    int    i50    = 0;
    int    i25    = 0;
    int    i5     = 0;

	if ((st != null) && (iDataType != null))
	{
	  iColor = new int[iDataType.length];
	  for (int i=0; i<iDataType.length; i++)
	  {
		iColor[i] = 0;
	  }

	  iMin = 1 + st.iCount / 5;
	  i50  = 1 + st.iCount / 2;
	  i25  = 1 + st.iCount / 4;
	  i5   = 1 + st.iCount / 20;

	  for (int i=0; i<iDataType.length; i++)
	  {
		if (st.checkData(iDataType[i]))
		{
		  data = st.getData(iDataType[i]);
		  if (data != null)
		  {
			iCount = 0;
		    for (int j=0; j<data.length; j++)
		    {
			  if (data[j] > 0.0)
			    iCount++;
			}

			if (iCount == st.iCount)
			  iColor[i] = 4;
			else if (iCount > i50)
			  iColor[i] = 3;
			else if (iCount > iMin)
			  iColor[i] = 2;
			else if (iCount > i5)
			  iColor[i] = 1;
		  }
		}
	  }
	}

	return (iColor);
  }

  /* ----------------------------------------------------------------------- *
   * ---------------------- GET STATISTICS METHODS ------------------------- *
   * ----------------------------------------------------------------------- */

//  public static final int ROWS    = 14;
  public static final int COLUMNS = 8;

  public static final int _MINIMUM = 0;
  public static final int _5       = 1;
  public static final int _25      = 2;
  public static final int _MEDIAN  = 3;
  public static final int _MEAN    = 4;
  public static final int _75      = 5;
  public static final int _95      = 6;
  public static final int _MAXIMUM = 7;

  /** Method getData()
   *  <p> This method will create the Averages Data
   *
   *  Columns -> Min, 5%, 25%,  Median, Mean, 75%, 95%, Max
   *
   *  @param  st     = Brine Data List Structure
   *  @param  iColor = Array of Brine Data Identifiers
   *  @return data   = array of averages for primary data
   */

  public static double[][] getData( fluidListStruct st, int iColor[] )
  {
	int i,j,k;
	double data[][] = null;
	double dTemp[]  = null;
	double dTemp1[] = null;
	double dSum     = 0.0;
	int    iTotal   = 0;
	int    iCount   = 0;
    int    iRows    = 0;

    if (st != null)
    {
	  iRows = iColor.length;
  	  data  = new double[iRows][COLUMNS];

	  for (i=0; i<iRows; i++)
	  {
	    for (j=0; j<COLUMNS; j++)
	    {
	  	  data[i][j] = 0.0;
	    }
	  }

      if (st.iCount > 0)
      {
        for (i=0; i<iRows; i++)
        {
    	  dTemp = new double[st.iCount];

		  for (k=0; k<st.iCount; k++)
		    dTemp[k] = 0.0;

		  if (st.checkData(iColor[i]))
		  {
		    dTemp = st.getData(iColor[i]);
		    dTemp = bubbleSort(dTemp);

            iTotal = 0;
		    for (k=0; k<st.iCount; k++)
		    {
			  if (dTemp[k] > 0.0)
			    iTotal++;
			}

            dTemp1 = new double[iTotal];

            iCount = 0;
            for (k=0; k<st.iCount; k++)
            {
			  if (dTemp[k] > 0.0)
			  {
  			    dTemp1[iCount] = dTemp[k];
  			    iCount++;
			  }
			}

            if (iCount > 0)
            {
              dTemp = new double[iCount];

		      for (k=0; k<iCount; k++)
		      {
			    dTemp[k] = dTemp1[k];
			  }

		      data[i][_MINIMUM] = dTemp[0];
		      data[i][_MAXIMUM] = dTemp[iCount-1];

              dSum = 0.0;
  		      for (k=0; k<iCount; k++)
		        dSum = dSum + dTemp[k];
		      data[i][_MEAN] = dSum / iCount;

		      data[i][_5]      = computePercentile(iCount, 0.05, dTemp);
		      data[i][_25]     = computePercentile(iCount, 0.25, dTemp);
		      data[i][_MEDIAN] = computePercentile(iCount, 0.5,  dTemp);
		      data[i][_75]     = computePercentile(iCount, 0.75, dTemp);
		      data[i][_95]     = computePercentile(iCount, 0.95, dTemp);
			}
		  }
	    }
	  }
	}

	return (data);
  }

  /** Method computePercentile()
   *  <p> This method will compute the Percentile value
   *  @param iTotal   = the Total number of rows
   *  @param dPercent = the decimal percentage value
   *  @param data     = array of numbers presorted
   */

  public static double computePercentile(
	                     int iTotal, double dPercent, double data[])
  {
	double dValue    = 0.0;
	double RowNum    = 0.0;
	double CelRowNum = 0.0;
	double FlrRowNum = 0.0;
	int    iFlr      = 0;
	int    iCel      = 0;

	if ((iTotal > 0) && data != null)
	{
	  RowNum    = 1.0 + (dPercent * (iTotal-1));
	  CelRowNum = Math.ceil(RowNum);
	  FlrRowNum = Math.floor(RowNum);

	  if ((CelRowNum == RowNum) && (FlrRowNum == RowNum))
	  {
		if (RowNum < data.length)
	      dValue = data[(int) RowNum];
	  }
	  else
	  {
		iFlr = (int) FlrRowNum;
		if (iFlr >= iTotal)
		  iFlr = iTotal-1;

		iCel = (int) CelRowNum;
		if (iCel >= iTotal)
		  iCel = iTotal-1;

		if ((iFlr < data.length) && (iCel < data.length))
		{
  	      dValue = (CelRowNum-RowNum) * data[iFlr] +
	               (RowNum-FlrRowNum) * data[iCel];
		}
	  }
	}

	return (dValue);
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  data = data array to be sorted
   * @return data = sorted data array
   */

  public static double[] bubbleSort(double data[])
  {
    boolean swappedOnPrevRun = true;
    double  dTemp   = 0.0;

    if (data != null)
    {
      if (data.length > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<data.length-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if(data[i] > data[i+1])
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              dTemp = data[i];

              // set element i+1 to where i used to be

              data[i] = data[i+1];

              // release the old i from temp into i+1 slot

              data[i+1] = dTemp;
            }
          }
        }
      }
    }

    return (data);
  }

  /* ======================================================================= *
   * ---------------------------- SET METHODS ------------------------------ *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will set the available curves.
   */

  public void setData(fluidListStruct st)
  {
	this.stBrine = fluid.fluidUtility.copyList( st );

    setStatistics();
    setButtons();
  }

  /** Method setData()
   *  <p> This method will set the available curves.
   */

  private void setData()
  {
	int i,j;
	int iCount = 0;

	if (stData != null)
	{
//	  if (stData.stBrine != null)
	  if (stBrine != null)
	  {
		for (i=0; i<stBrine.iCount; i++)
		{
		  if (iEnd < stBrine.stItem[i].group_no)
		    iEnd = stBrine.stItem[i].group_no;
		}

        for (i=0; i<TOTAL; i++)
        {
		  for (j=0; j<iAllowed.length; j++)
		  {
			if (iAllowed[j] == i)
			{
//		      if (stData.stBrine.checkData(i))
		      if (stBrine.checkData(i))
		      {
			    iTotal++;
//System.out.println(i+" "+iAllowed[j]+" "+iTotal);
			  }
		    }
		  }
		}

		if (iTotal > 0)
		{
		  iDataType = new int[iTotal];

          for (i=0; i<TOTAL; i++)
          {
		    for (j=0; j<iAllowed.length; j++)
		    {
		  	  if (iAllowed[j] == i)
			  {
//		        if (stData.stBrine.checkData(i))
		        if (stBrine.checkData(i))
		        {
				  if (iCount < iTotal)
				  {
				    iDataType[iCount] = i;
//System.out.println(iCount+" "+iDataType[iCount]);
				    iCount++;
				  }
		  	    }
		      }
		    }
	  	  }
	    }

	    iColor = getColor( stBrine, iDataType );
	  }
	}

	if (iEnd > 0)
	  iEnd++;
  }

  /** Method setButtons()
   *  <p> This method will enable or disable buttons depending on the data
   */

  private void setButtons()
  {
    int iR = 0;
    int iB = 0;
    int iG = 0;

	if (iTotal > 0)
	{
	  for (int i=0; i<iTotal; i++)
	  {
		if (cb[i][0].isSelected()) { iR = 1; }
		if (cb[i][1].isSelected()) { iG = 1; }
		if (cb[i][2].isSelected()) { iB = 1; }
	  }

	  for (int i=0; i<iTotal; i++)
	  {
		if (iR > 0) { cb[i][0].setEnabled(false); }
		if (iG > 0) { cb[i][1].setEnabled(false); }
		if (iB > 0) { cb[i][2].setEnabled(false); }
	  }
	}
  }

  /** Method clear()
   *  <p> This method will clear the selections
   */

  private void clear()
  {
    iRed   = -1;
    iGreen = -1;
    iBlue  = -1;

	if (iTotal > 0)
	{
	  for (int i=0; i<iTotal; i++)
	  {
	    for (int j=0; j<3; j++)
	    {
		  cb[i][j].setSelected(false);
		  cb[i][j].setEnabled(true);
		}
	  }
	}

    if (notifier != null)
      notifier.notifyObservers( new String("RGB Curves NOT Selected") );

	pTable.clear();
  }

  /** Method setStatistics()
   *  <p> This method will set the Statistics data for curves selected
   */

  private void setStatistics()
  {
	int    iCURVES[] = null;
	int    iCount    = 0;
    double data[][]  = null;
    double dRow[]    = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    String sMNEM[]   = { "", "", "" };
    double dMin      = 0.0;
    double dMax      = 0.0;
    int    iRow      = 0;

	if (pTable != null)
	{
	  if (iRed > -1)
	    iCount++;

	  if (iGreen > -1)
	    iCount++;

	  if (iBlue > -1)
	    iCount++;

      if (iCount > 0)
      {
		iCURVES = new int[iCount];
		iCount  = 0;

	    if (iRed > -1)
	    {
		  iCURVES[iCount] = iRed;
		  sMNEM[iCount]   = new String( fluid.fluidStandardTools.CURVES[iRed][1] );
	      iCount++;
		}

	    if (iGreen > -1)
	    {
		  iCURVES[iCount] = iGreen;
		  sMNEM[iCount]   = new String( fluid.fluidStandardTools.CURVES[iGreen][1] );
	      iCount++;
		}

	    if (iBlue > -1)
	    {
		  iCURVES[iCount] = iBlue;
		  sMNEM[iCount]   = new String( fluid.fluidStandardTools.CURVES[iBlue][1] );
	      iCount++;
		}

        data = getData( stBrine, iCURVES );

        for (int i=0; i<iCount; i++)
        {
		  for (int j=0; j<8; j++)
		    dRow[j] = data[i][j];

		  pTable.setRow( i, 3, dRow, sMNEM[i] );

          dMin = (double) ((int) (dRow[1] * 10.0) / 10.0);
          dMax = (double) ((int) (dRow[6] * 10.0) / 10.0);

          if (iRed   > -1) { iRow = 0; }
          if (iGreen > -1) { iRow = 1; }
          if (iBlue  > -1) { iRow = 2; }

          sColor[iRow]    = new String(sMNEM[i]);
          dColor[iRow][0] = dMin;
          dColor[iRow][1] = dMax;

  	    }
	  }
	}

    if ((iRed > -1) && (iGreen > -1) && (iBlue > -1))
    {
      if (notifier != null)
        notifier.notifyObservers( new String("RGB Curves Selected") );
	}
  }

  /* ======================================================================= *
   * -------------------------- PLOT MAP ACTIONS --------------------------- *
   * ======================================================================= */

  /** Method initData()
   *  <p> This method will build the plot mapping data structure for plotting
   *  @return st = grid Plot Mapping Structure
   */

  private gridPlotMappingStruct initData()
  {
 	gridPlotMappingStruct st = null;
    int    iTotal    = 0;
    int    iCount    = 0;
    double dXMin     = 0.0;
    double dYMin     = 0.0;
    double data[][]  = null;
	int    iCURVES[] = { -1, -1, -1 };

    if (stData != null)
    {
	  iTotal = stData.stWellList.iCount;
      st     = new gridPlotMappingStruct();

      // Well Data to plot on top of the mapping plot.

      st.iWells     = iTotal; // Total Number of Wells that will be displayed
      st.dX         = new double[iTotal]; // X - Position of Well
      st.dY         = new double[iTotal]; // Y - Position of Well
      st.dZR        = new double[iTotal]; // Z - Data Value of Well - Red
      st.dZG        = new double[iTotal]; // Z - Data Value of Well - Green
      st.dZB        = new double[iTotal]; // Z - Data Value of Well - Blue
      st.status     = new String[iTotal]; // Well Status Symbol;
      st.sKID       = new String[iTotal]; // Well KID;
      st.sAPI       = new String[iTotal]; // Well API-Number;
      st.sLease     = new String[iTotal]; // Well Lease Name;
      st.iTriangle  = new int[iTotal];    // Zone has been perforated.

      // Limits to compute Color

	  st.iRed     = iRed;   // Indicator if Curve is present
	  if (iRed > -1)
	  {
	    st.dMinR  = dColor[0][0];
	    st.dMaxR  = dColor[0][1];
	  }

	  st.iGreen   = iGreen;   // Indicator if Curve is present
	  if (iGreen > -1)
	  {
	    st.dMinG  = dColor[1][0];
	    st.dMaxG  = dColor[1][1];
	  }

      st.iBlue    = iBlue;   // Indicator if Curve is present
      if (iBlue > -1)
      {
	    st.dMinB  = dColor[2][0];
	    st.dMaxB  = dColor[2][1];
	  }

      for (int i=0; i<iTotal; i++)
      {
        st.dX[i]        = 0.0;
        st.dY[i]        = 0.0;
        st.dZR[i]       = 0.0;
        st.dZG[i]       = 0.0;
        st.dZB[i]       = 0.0;
        st.sKID[i]      = new String("");
        st.sAPI[i]      = new String("");
        st.sLease[i]    = new String("");
        st.status[i]    = new String("");
        st.iTriangle[i] = 0;
	  }

	  for (int k=0; k<stData.stWellList.iCount; k++)
	  {
        // Find the Minimum UTM-X and UTM-Y Value.

        if (k == 0)
        {
          dXMin = stData.stWellList.stItem[k].dUTMx;
          dYMin = stData.stWellList.stItem[k].dUTMy;
        }

        if (dXMin > stData.stWellList.stItem[k].dUTMx)
          dXMin = stData.stWellList.stItem[k].dUTMx;

        if (dYMin > stData.stWellList.stItem[k].dUTMy)
          dYMin = stData.stWellList.stItem[k].dUTMy;
	  }

      st.dXo = 3.3 * (st.dUTMx - dXMin);
      st.dYo = 3.3 * (st.dUTMy - dYMin);

	  for (int k=0; k<stData.stWellList.iCount; k++)
	  {
        st.dX[k]        = 3.3 * (stData.stWellList.stItem[k].dUTMx - dXMin);
        st.dY[k]        = 3.3 * (stData.stWellList.stItem[k].dUTMy - dYMin);
        st.dZR[k]       = 0.0;
        st.dZG[k]       = 0.0;
        st.dZB[k]       = 0.0;
        st.sKID[k]      = new String(stData.stWellList.stItem[k].sWELLKID);
        st.sAPI[k]      = new String(stData.stWellList.stItem[k].sAPI);
        st.sLease[k]    = new String(stData.stWellList.stItem[k].sName);
        st.status[k]    = new String(stData.stWellList.stItem[k].status);
        st.iTriangle[k] = 0;
	  }

      // Set Statistics Data

      iCount = 0;

 	  if (iRed   > -1)
 	  {
		iCURVES[iCount] = iRed;
		iCount++;
	  }

	  if (iGreen > -1)
	  {
		iCURVES[iCount] = iGreen;
		iCount++;
	  }

	  if (iBlue  > -1)
	  {
		iCURVES[iCount] = iBlue;
		iCount++;
	  }

      st.data = getData( stBrine, iCURVES );
	}

	return (st);
  }


  /** Method setData()
   *  <p> This method will load the plot mapping data structure for plotting
   *  @param  iStep = the group number
   *  @param  st = grid Plot Mapping Structure
   *  @return st = grid Plot Mapping Structure
   */

  private gridPlotMappingStruct setData( int iStep, gridPlotMappingStruct st)
  {
    double data[][]  = null;

	if (st != null)
	{
      if (stBrine != null)
      {
		for (int i=0; i<st.iWells; i++)
		{
		  for (int j=0; j<stBrine.iCount; j++)
		  {
			if ((iStep == stBrine.stItem[j].group_no) && (st.sKID[i].equals(stBrine.stItem[j].sWellKID)))
			{
    	 	  st.sValue = new String(stBrine.stItem[j].sRecovery);
    	 	  st.sZone  = new String("");

              st.dZR[i] = stBrine.getData( st.iRed,   j );
              st.dZG[i] = stBrine.getData( st.iGreen, j );
              st.dZB[i] = stBrine.getData( st.iBlue,  j );
			}
		  }
		}
	  }

      // Set the Grid for the Well Data

      pGridding = new gridMath( st.iWells, st.dX, st.dY );

      if (stGrid != null)
        pGridding.setParametersStructData( stGrid );
      else
        pGridding.setGridCellSpacing( pGridding.getGridCellSpacing()/5 );

      // Get Gridding Paramters Data Structure for Output

      stGrid = grid.gridDataUtility.copyGrid( pGridding.getParametersStructData() );

      // Well Plotting area Dimensions

	  st.dXMin      = pGridding.getMinimumX();        // Minimum X Position
	  st.dXMax      = pGridding.getMaximumX();        // Maximum X Position
	  st.dXCellSize = pGridding.getGridCellSpacing(); // X Cell Size
	  st.dYMin      = pGridding.getMinimumY();        // Minimum X Position
	  st.dYMax      = pGridding.getMaximumY();        // Maximum X Position
	  st.dYCellSize = pGridding.getGridCellSpacing(); // Y Cell Size

      // Get the Brine Data for Plotting

      data          = pGridding.fillGrid( st.dZR );
      st.iRows      = pGridding.getGridRows();
      st.iColumns   = pGridding.getGridColumns();

      st.dataR = new double[st.iRows][st.iColumns];
      for (int i=0; i<st.iRows; i++)
      {
        for (int j=0; j<st.iColumns; j++)
        {
	      if (data[i][j] > 0)
            st.dataR[i][j] = data[i][j];
          else
            st.dataR[i][j] = 0.0;
        }
      }

      data = pGridding.fillGrid( st.dZG );

      st.dataG = new double[st.iRows][st.iColumns];
      for (int i=0; i<st.iRows; i++)
      {
        for (int j=0; j<st.iColumns; j++)
        {
		  if (data[i][j] > 0)
            st.dataG[i][j] = data[i][j];
          else
            st.dataG[i][j] = 0.0;
        }
      }

      data = pGridding.fillGrid( st.dZB );

      st.dataB = new double[st.iRows][st.iColumns];
      for (int i=0; i<st.iRows; i++)
      {
        for (int j=0; j<st.iColumns; j++)
        {
		  if (data[i][j] > 0)
            st.dataB[i][j] = data[i][j];
          else
            st.dataB[i][j] = 0.0;
        }
      }
	}

	return (st);
  }

  public gridPlotMappingListStruct getData()
  {
	gridPlotMappingListStruct st = null;

    if ((iRed > -1) && (iGreen > -1) && (iBlue > -1) && (iEnd > 0))
    {
	  st        = new gridPlotMappingListStruct();
      st.stItem = new gridPlotMappingStruct[iEnd];
      st.iCount = iEnd;

      for (int i=0; i<st.iCount; i++)
      {
		st.stItem[i] = initData();
        st.stItem[i] = setData( i, st.stItem[i] );
	  }
	}

	return (st);
  }

  /* ======================================================================= *
   * ------------------------------ ACTIONS -------------------------------- *
   * ======================================================================= */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions for this Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnClear) { clear(); }

	if (iTotal > 0)
	{
	  for (int i=0; i<iTotal; i++)
	  {
	    for (int j=0; j<3; j++)
	    {
		  if (event.getSource() == cb[i][j])
		  {
			if (j==0) { iRed   = iDataType[i]; }
			if (j==1) { iGreen = iDataType[i]; }
			if (j==2) { iBlue  = iDataType[i]; }
		  }
		}
	  }

	  setStatistics();
	}

    setButtons();
  }
}

/*
 *  @version 1.1 02/26/2016
 *  @author  John Victorine
 */
