/*
 * @gridMath.java Version 1.1 05/14/2002
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.math;

import java.awt.*;
import javax.swing.*;

import grid.math.grid;
import grid.math.gridParameters;
import grid.math.gridSearchParameters;
import grid.math.gridParametersStruct;

/** Class gridMath
 *  <p> This Class will create a grid of the data passed in.
 *
 *  @version 1.1 05/14/2002
 *  @author  John R. Victorine
 */

public class gridMath
{
  private int    iXCells          = 0;
  private int    iYCells          = 0;
  private double dCellSize        = 0.0;
  private int    iNeighbors       = 8;
  private double dMaxDistTo1st    = 0.0;
  private double dMaxSearchRadius = 0.0;
  private double dExponent        = 2.0;
  private double dXMax            = 0.0;
  private double dXMin            = 0.0;
  private double dYMax            = 0.0;
  private double dYMin            = 0.0;

  private int    iRows            = 0;
  private double dX[]             = null;
  private double dY[]             = null;

  private int    ix               = 0;
  private int    iy               = 0;
  private double data[][]         = null;

  private gridParametersStruct stParameters = new gridParametersStruct();

  /** Constructor gridMath()
   *  <p> This is the Constructor for this class.
   * @param rows = The number of rows
   * @param x    = array of X locations
   * @param y    = array of Y locations
   */

  public gridMath(int rows, double x[], double y[])
  {
    iRows = rows;

    if (iRows > 0)
    {
      dX = new double[iRows];
      dY = new double[iRows];

      for (int i=0; i<iRows; i++)
      {
        dX[i] = x[i];
        dY[i] = y[i];
      }

      createGrid();
      computeSearchParameters();
    }
  }

  /** Method delete()
   * <p> This method will set the string variable to null and initialize all the
   *     other variables.
   */

  public void delete()
  {
    iXCells          = 0;
    iYCells          = 0;
    dCellSize        = 0.0;
    iNeighbors       = 8;
    dMaxDistTo1st    = 0.0;
    dMaxSearchRadius = 0.0;
    dExponent        = 2.0;
    dXMax            = 0.0;
    dXMin            = 0.0;
    dYMax            = 0.0;
    dYMin            = 0.0;

    iRows            = 0;
    dX               = null;
    dY               = null;

    ix               = 0;
    iy               = 0;
    data             = null;

    if (stParameters != null)
      stParameters.delete();
    stParameters = null;
  }

  /** Method createGrid()
   *  <p> This Method will create the Grid.
   */

  private void createGrid()
  {
    int    i             = 0;
    int    j             = 0;
    double dXo           = 0.0;
    double dYo           = 0.0;
    double dXMinDistance = 0.0;
    double dYMinDistance = 0.0;
    int    iSet          = 0;

    for (i=0; i<iRows; i++)
    {
      if (iSet == 0)
      {
        dXMinDistance = Math.abs(dX[i] - dX[i+1]);
        dYMinDistance = Math.abs(dY[i] - dY[i+1]);
        iSet = 1;
      }

      for (j=0; j<iRows; j++)
      {
        if (i!=j)
        {
          dXo = Math.abs(dX[i] - dX[j]);
          dYo = Math.abs(dY[i] - dY[j]);

          if (dXo > dXMinDistance)
            dXMinDistance = dXo;

          if (dYo > dYMinDistance)
            dYMinDistance = dYo;
        }
      }
    }

    if (dXo > dYo)
      dCellSize = Math.ceil(dYMinDistance / 5);
    else
      dCellSize = Math.ceil(dXMinDistance / 5);

    for (i=0; i<iRows; i++)
    {
      if (i==0)
      {
        dXMin = Math.floor(dX[i]);
        dXMax = Math.ceil(dX[i]);
        dYMin = Math.floor(dY[i]);
        dYMax = Math.ceil(dY[i]);
      }

      if (dXMin > dX[i])
        dXMin = Math.floor(dX[i]);

      if (dXMax < dX[i])
        dXMax = Math.ceil(dX[i]);

      if (dYMin > dY[i])
        dYMin = Math.floor(dY[i]);

      if (dYMax < dY[i])
        dYMax = Math.ceil(dY[i]);
    }

    dXMax = dXMax + 4.0 * dCellSize;
    dXMin = dXMin - 4.0 * dCellSize;
    dYMax = dYMax + 4.0 * dCellSize;
    dYMin = dYMin - 4.0 * dCellSize;
  }

  /** Method computeSearchParameters()
   *  <p> This mehtod will compute the search parameters from the data set.
   */

  private void computeSearchParameters()
  {
    double dXo = 0.0;
    double dYo = 0.0;

    iNeighbors = 2;
    if ((iRows < 9) && (iRows > 1))
      iNeighbors = iRows;
    else
      iNeighbors = 8;

    // Get the Maximum Distance to Nearest Data Point & Maximum Search Radius.

    dXo = dXMax - dXMin;
    dYo = dYMax - dYMin;

    if (dYo < dXo)
    {
      dMaxDistTo1st    = Math.ceil(dYo / 6.0);
      dMaxSearchRadius = Math.ceil(dYo / 3.0);
    }
    else
    {
      dMaxDistTo1st    = Math.ceil(dXo / 6.0);
      dMaxSearchRadius = Math.ceil(dXo / 3.0);
    }

    iXCells = 1 + (int) (dXo / dCellSize);
    iYCells = 1 + (int) (dYo / dCellSize);
  }

   /** Method fillGrid()
    *  <p> This method will compute the individual cells value.
    *  @param dZ = The data array to be gridded
    */

   public double[][] fillGrid(double dZ[])
   {
     int    i;
     int    j;

     int nx = (int) ((dXMax - dXMin) / dCellSize);
     int ny = (int) ((dYMax - dYMin) / dCellSize);

     grid pGrid = new grid( dX,
                            dY,
                            dZ,
                            new gridParameters( dXMin,
                                                dXMax,
                                                nx,
                                                dYMin,
                                                dYMax,
                                                ny ),
		                    new gridSearchParameters( iNeighbors,
                                                      dMaxDistTo1st,
                                                      dMaxSearchRadius,
                                                      dExponent ));

     ix   = pGrid.GridPar.nx();
     iy   = pGrid.GridPar.ny();
     data = new double[ix][iy];

     for (i=0; i<ix; i++)
     {
       for (j=0; j<iy; j++)
       {
         data[i][j] = pGrid.zValues[i][j];
       }
     }

     return (data);
   }

   /* ===================================================================== *
    * ---------------------------- GET METHODS ---------------------------- *
    * ===================================================================== */

  /** getGridRows()
   *  <p> This method will return the number of grid rows
   *  @return ix = Number of Grid Rows
   */

  public int getGridRows() { return (ix); }

  /** getGridColumns()
   *  <p> This method will return the number of grid columns
   *  @return iy = Number of Grid Columns
   */

  public int getGridColumns() { return (iy); }

  /** getGridData()
   *  <p> This method will return the grid data.
   *  @return data = the array of grid data.
   */

  public double[][] getGridData() {return (data); }

  /** getNumberOfRows()
   *  <p> This method will return the number of Rows of the Grid.
   *  @return iXCells = Number of Rows
   */

  public int getNumberOfRows() { return (iXCells); }

  /** getNumberOfColumns()
   *  <p> This method will return the number of Columns of the Grid.
   *  @return iYCells = Number of Columns
   */

  public int getNumberOfColumns() { return (iYCells); }

  /** getGridCellSpacing()
   *  <p> This method will return the Grid Cell Spacing
   *  @return dCellSize = Grid Cell Spacing
   */

  public double getGridCellSpacing() { return (dCellSize); }

  /** getNumberOfNearestNeighbors()
   *  <p> This method will return the Number of Nearest Neighbors
   *  @return iNeighbors = Number of Nearest Neighbors
   */

  public int getNumberOfNearestNeighbors() { return (iNeighbors); }

  /** getMaxDistanceToNearestDataPoint()
   *  <p> This method will return the Maximum Distance to Nearest Data Point
   *  @return dMaxDistTo1st = Maximum Distance to Nearest Data Point
   */

  public double getMaxDistanceToNearestDataPoint() { return (dMaxDistTo1st); }

  /** getMaxSearchRadius()
   *  <p> This method will return the Maximum Search Radius
   *  @return dMaxSearchRadius = Maximum Search Radius
   */

  public double getMaxSearchRadius() { return (dMaxSearchRadius); }

  /** getInversDistanceExponent()
   *  <p> This method will return the Inverse Distance Weighting Exponent
   *  @return dExponent = Inverse Distance Weighting Exponent
   */

  public double getInversDistanceExponent() { return (dExponent); }

  /** getMaximumX()
   *  <p> This method will return the Maximum X in Feet.
   *  @return dXMax = Maximum X in Feet.
   */

  public double getMaximumX() { return (dXMax); }

  /** getMinimumX()
   *  <p> This method will return the Minimum X in Feet.
   *  @return dXMax = Minimum X in Feet.
   */

  public double getMinimumX() { return (dXMin); }

  /** getMaximumY()
   *  <p> This method will return the Maximum X in Feet.
   *  @return dXMax = Maximum X in Feet.
   */

  public double getMaximumY() { return (dYMax); }

  /** getMinimumY()
   *  <p> This method will return the Minimum X in Feet.
   *  @return dXMax = Minimum X in Feet.
   */

  public double getMinimumY() { return (dYMin); }

  /** Method getParametersStructData()
   *  <p> This method will retrieve the Parameters Structure Data.
   * @return stParameters = The Gridding Parameters Data Structure
   */

  public gridParametersStruct getParametersStructData()
  {
    stParameters.iXCells          = iXCells;
    stParameters.iYCells          = iYCells;
    stParameters.dCellSize        = dCellSize;
    stParameters.iNeighbors       = iNeighbors;
    stParameters.dMaxDistTo1st    = dMaxDistTo1st;
    stParameters.dMaxSearchRadius = dMaxSearchRadius;
    stParameters.dExponent        = dExponent;
    stParameters.dXMax            = dXMax;
    stParameters.dXMin            = dXMin;
    stParameters.dYMax            = dYMax;
    stParameters.dYMin            = dYMin;

    return (stParameters);
  }

  /* ===================================================================== *
   * ---------------------------- SET METHODS ---------------------------- *
   * ===================================================================== */

  /** setGridCellSpacing()
   *  <p> This method will set the Grid Cell Spacing
   *  @param cellSize = Grid Cell Spacing
   */

  public void setGridCellSpacing(double cellSize)
  {
    dXMax = dXMax - 4.0 * dCellSize;
    dXMin = dXMin + 4.0 * dCellSize;
    dYMax = dYMax - 4.0 * dCellSize;
    dYMin = dYMin + 4.0 * dCellSize;

    dCellSize = cellSize;

    dXMax = dXMax + 4.0 * dCellSize;
    dXMin = dXMin - 4.0 * dCellSize;
    dYMax = dYMax + 4.0 * dCellSize;
    dYMin = dYMin - 4.0 * dCellSize;

    computeSearchParameters();
  }

  /** Method setParametersStructData()
   *  <p> This method will set the Parameters Structure data
   * @param st = The Gridding Parameters Data Structure
   */

  public void setParametersStructData(gridParametersStruct st)
  {
    stParameters     = st;

    iXCells          = stParameters.iXCells;
    iYCells          = stParameters.iYCells;
    dCellSize        = stParameters.dCellSize;
    iNeighbors       = stParameters.iNeighbors;
    dMaxDistTo1st    = stParameters.dMaxDistTo1st;
    dMaxSearchRadius = stParameters.dMaxSearchRadius;
    dExponent        = stParameters.dExponent;
    dXMax            = stParameters.dXMax;
    dXMin            = stParameters.dXMin;
    dYMax            = stParameters.dYMax;
    dYMin            = stParameters.dYMin;
  }
}
