/*
 * @(#)gridParameters.java Version 1.00, 28 June 2001
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, USA
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of the
 * Kansas Geological Survey.  ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with the Kansas Geological Survey.
 *
 *       Last change:  GCB  28 Jun 2001    9:35 am
 *                     JRV  24 Nov 2003    2:36 pm
 *                     - Changed the location to grid.math from pfeffer.math
 *                       and name of the file from pfefferGridParameters.java
 *                       to gridParameters.java. Modified the comments to match
 *                       GEMINI Standards.
 */

package grid.math;

public class gridParameters
{
  /**
   * Defines grid specification parameters for a rectangular X,Y grid.
   * Grid parameters are private: "set" routines used to maintain consistent
   * grid specification.
   *
   * @version 1.00, 28 June 2001
   * @author Geoff Bohling
   */

  /** X coordinate of first grid node */

  private double XMin;

  /** X coordinate of last grid node  */

  private double XMax;

  /** X-direction grid increment      */

  private double DX;

  /** Number of nodes in X direction  */

  private int    NX;

  /** Y coordinate of first grid node */

  private double YMin;

  /** Y coordinate of last grid node  */

  private double YMax;

  /** Y-direction grid increment      */

  private double DY;

  /** Number of nodes in Y direction  */

  private int    NY;

 /** Constructor gridParameters()
  * <p>using min, max, increment; sets number of nodes
  * @param xminin = X coordinate of first grid node
  * @param xmaxin = X coordinate of last grid node
  * @param dxin   = X Direction Grid Increment
  * @param yminin = Y coordinate of first grid node
  * @param ymaxin = Y coordinate of last grid node
  * @param dyin   = Y Direction grid increment
  */

  public gridParameters( double xminin,
                         double xmaxin,
                         double dxin,
	                     double yminin,
                         double ymaxin,
                         double dyin )
  {
    XMin = xminin;
    DX   = dxin;
    NX   = 1 + (int) ( 0.5 + (xmaxin-XMin)/DX );
    XMax = XMin + (NX-1)*DX;

    YMin = yminin;
    DY   = dyin;
    NY   = 1 + (int) ( 0.5 + (ymaxin-YMin)/DY );
    YMax = YMin + (NY-1)*DY;
  }

  /** Constructor using min, max, number of nodes; sets increment */

  /** Constructor gridParameters()
   * <p>using min, max, number of nodes; sets increment
   * @param xminin = X coordinate of first grid node
   * @param xmaxin = X coordinate of last grid node
   * @param nxin   = Number of nodes in X direction
   * @param yminin = Y coordinate of first grid node
   * @param ymaxin = Y coordinate of last grid node
   * @param nyin   = Number of nodes in Y direction
   */

  public gridParameters( double xminin,
                         double xmaxin,
                         int nxin,
	                     double yminin,
                         double ymaxin,
                         int nyin )
  {
    XMin = xminin;
    NX = nxin;
    DX = (xmaxin-XMin)/(NX-1);
    XMax = XMin + (NX-1)*DX;

    YMin = yminin;
    NY = nyin;
    DY = (ymaxin-YMin)/(NY-1);
    YMax = YMin + (NY-1)*DY;
  }

  /** Constructor gridParameters()
   * <p>Constructor based on x, y locations of data points; sets up
   *    reasonable default grid
   * @param x = Array of x locations
   * @param y = Array of y locations
   */

  public gridParameters( double[] x, double[] y )
  {
    // Find minimum, maximum x and y coordinates:

    double xmind = x[0];
    double ymind = y[0];
    double xmaxd = x[0];
    double ymaxd = y[0];

    for ( int i=1; i<x.length; i++)
    {
      if ( x[i] < xmind ) xmind = x[i];
      if ( y[i] < ymind ) ymind = y[i];
      if ( x[i] > xmaxd ) xmaxd = x[i];
      if ( y[i] > ymaxd ) ymaxd = y[i];
    }

    // Grid limits from expanding 1% of data range on each end:

    XMin = xmind - 0.01*(xmaxd-xmind);
    YMin = ymind - 0.01*(ymaxd-ymind);
    XMax = xmaxd + 0.01*(xmaxd-xmind);
    YMax = ymaxd + 0.01*(ymaxd-ymind);

    // Put 101 nodes (100 increments) in longer direction; set other
    // increment equal to first:

    if ( XMax-XMin > YMax-YMin )
    {
      NX = 101;
      DX = (XMax-XMin)/(NX-1);
      DY = DX;
      NY = 1 + (int) ( 0.5 + (YMax-YMin)/DY );
    }
    else
    {
      NY = 101;
      DY = (YMax-YMin)/(NY-1);
      DX = DY;
      NX = 1 + (int) ( 0.5 + (XMax-XMin)/DX );
    }

    // Adjust XMax, YMax for consistent specification:

    XMax = XMin + (NX-1)*DX;
    YMax = YMin + (NY-1)*DY;
  }

  /** Constructor gridParameters()
   * <p>Constructor based on x,y coordinates of data points with
   *    number of nodes specified
   * @param x    = Array of x locations
   * @param y    = Array of y locations
   * @param nxin = Number of nodes in X direction
   * @param nyin = Number of nodes in Y direction
   */

  public gridParameters( double[] x, double[] y, int nxin, int nyin )
  {
    // Find minimum, maximum x and y coordinates:

    double xmind = x[0];
    double ymind = y[0];
    double xmaxd = x[0];
    double ymaxd = y[0];

    for ( int i=1; i<x.length; i++)
    {
      if ( x[i] < xmind ) xmind = x[i];
      if ( y[i] < ymind ) ymind = y[i];
      if ( x[i] > xmaxd ) xmaxd = x[i];
      if ( y[i] > ymaxd ) ymaxd = y[i];
    }

    // Expand by 1% on each end:

    XMin = xmind - 0.01*(xmaxd-xmind);
    YMin = ymind - 0.01*(ymaxd-ymind);
    XMax = xmaxd + 0.01*(xmaxd-xmind);
    YMax = ymaxd + 0.01*(ymaxd-ymind);

    // Use specified number of nodes in each direction:

    NX = nxin;
    NY = nyin;

    // Get corresponding incremants:

    DX = (XMax-XMin)/(NX-1);
    DY = (YMax-YMin)/(NY-1);

    // Adjust XMax, YMax for consistency:

    XMax = XMin + (NX-1)*DX;
    YMax = YMin + (NY-1)*DY;
  }

  /* ===================================================================== *
   * ---------------------------- SET METHODS ---------------------------- *
   * ===================================================================== */

  /** Method setXmin()
   * <p> Sets XMin; adjusts NX, DX to match
   * @param xminin = X coordinate of first grid node
   */

  public void setXmin( double xminin )
  {
    XMin = xminin;
    NX = 1 + (int) ( 0.5 + (XMax-XMin)/DX );
    DX = (XMax-XMin)/(NX-1);
  }

  /** Method setYmin()
   * <p> Sets YMin; adjusts NY, DY to match
   * @param yminin = Y coordinate of first grid node
   */

  public void setYmin( double yminin )
  {
    YMin = yminin;
    NY = 1 + (int) ( 0.5 + (YMax-YMin)/DY );
    DY = (YMax-YMin)/(NY-1);
  }

  /** Method setXmax()
   * <p> Sets XMax; adjusts NX, DX to match
   * @param xmaxin = X coordinate of last grid node
   */

  public void setXmax( double xmaxin )
  {
    XMax = xmaxin;
    NX = 1 + (int) ( 0.5 + (XMax-XMin)/DX );
    DX = (XMax-XMin)/(NX-1);
  }

  /** Method setYmax()
   * <p> Sets YMax; adjusts NY, DY to match
   * @param ymaxin = Y coordinate of last grid node
   */

  public void setYmax( double ymaxin )
  {
    YMax = ymaxin;
    NY = 1 + (int) ( 0.5 + (YMax-YMin)/DY );
    DY = (YMax-YMin)/(NY-1);
  }

  /** Method setDx()
   * <p> Sets DX; adjusts NX, XMax to match
   * @param dxin   = X Direction Grid Increment
   */

  public void setDx( double dxin )
  {
    DX = dxin;
    NX = 1 + (int) ( 0.5 + (XMax-XMin)/DX );
    XMax = XMin + (NX-1)*DX;
  }

  /** Method setDy()
   * <p> Sets DY; adjusts NY, YMax to match
   * @param dyin   = Y Direction Grid Increment
   */

  public void setDy( double dyin )
  {
    DY = dyin;
    NY = 1 + (int) ( 0.5 + (YMax-YMin)/DY );
    YMax = YMin + (NY-1)*DY;
  }

  /** Method setNx()
   * <p> Sets NX; adjusts DX, XMax to match
   * @param nxin = Number of nodes in X direction
   */

  public void setNx( int nxin )
  {
    NX = nxin;
    DX = (XMax-XMin)/(NX-1);
    XMax = XMin + (NX-1)*DX;
  }

  /** Method setNy()
   * <p> Sets NY; adjusts DY, YMax to match
   * @param nyin = Number of nodes in Y direction
   */

  public void setNy( int nyin )
  {
    NY = nyin;
    DY = (YMax-YMin)/(NY-1);
    YMax = YMin + (NY-1)*DY;
  }

  /* ===================================================================== *
   * -------------------------- RETURN METHODS --------------------------- *
   * ===================================================================== */

  /** Method xmin()
   * <p> Returns XMin - X coordinate of first grid node
   * @return XMin = X coordinate of first grid node
   */

  public double xmin() {return XMin;}

  /** Method xmax()
   * <p> Returns XMax - X coordinate of last grid node
   * @return XMin = X coordinate of last grid node
   */

  public double xmax() {return XMax;}

  /** Method dx()
   * <p> Returns DX - X-direction grid increment
   * @return DX = X-direction grid increment
   */

  public double dx()   {return DX;}

  /** Method nx()
   * <p> Returns NX - Number of nodes in X direction
   * @return NX = Number of nodes in X direction
   */

  public int    nx()   {return NX;}

  /** Method ymin()
   * <p> Returns YMin - Y coordinate of first grid node
   * @return YMin = Y coordinate of first grid node
   */

  public double ymin() {return YMin;}

  /** Method ymax()
   * <p> Returns YMax - Y coordinate of last grid node
   * @return YMin = Y coordinate of last grid node
   */

  public double ymax() {return YMax;}

  /** Method dy()
   * <p> Returns DY - Y-direction grid increment
   * @return DY = Y-direction grid increment
   */

  public double dy()   {return DY;}

  /** Method ny()
   * <p> Returns NY - Number of nodes in Y direction
   * @return NY = Number of nodes in Y direction
   */

  public int    ny()   {return NY;}
}
