/*
 * @(#)gridSearchParameters.java Version 1.00, 28 June 2001
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, USA
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of the
 * Kansas Geological Survey.  ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with the Kansas Geological Survey.
 *
 *	Last change:  GCB  28 Jun 2001    9:48 am
 *                    JRV  25 Nov 2003    6:39 am
 *                    - Changed the location to grid.math from pfeffer.math
 *                      and name of the file from pfefferSearchParameters.java
 *                      to gridSearchParameters.java. Modified the comments to
 *                      match GEMINI Standards.
 */

package grid.math;

/**
 * Defines nearest-neighbor search parameters and exponent for inverse-distance
 * interpolation.
 *
 * @version 1.00, 20 June 2001
 * @author Geoff Bohling
 */

public class gridSearchParameters
{
  /** The number of nearest neighbors to use */

  private int NNN;

  /** The maximum allowed distance to the first nearest neighbor */

  private double Rad1stMax;

  /** The maximum search radius */

  private double RadSrchMax;

  /** The power of distance to use */

  private double DExpo;

  /** Constructor gridSearchParameters()
   * <p>Constructor taking the four parameter values
   * @param n    = The number of nearest neighbors to use
   * @param rad1 = The maximum allowed distance to the first nearest neighbor
   * @param rad2 = The maximum search radius
   * @param expo = The power of distance to use
   */

  public gridSearchParameters(int n, double rad1, double rad2, double expo)
  {
    NNN = n;
    Rad1stMax = rad1;
    RadSrchMax = rad2;
    DExpo = expo;
  }

  /** Constructor gridSearchParameters()
   * <p>Constructor based on x,y coordinate vectors; sets reasonable defaults
   * @param x    = The x coordinate vector
   * @param y    = The y coordinate vector
   */

  public gridSearchParameters(double[] x, double[] y)
  {
    // Find ranges of x,y coordinate values:

    double xmin = x[0];
    double ymin = y[0];
    double xmax = x[0];
    double ymax = y[0];
    int    n = x.length;

    for ( int i=1; i<n; i++)
    {
      if ( x[i] < xmin ) xmin = x[i];
      if ( y[i] < ymin ) ymin = y[i];
      if ( x[i] > xmax ) xmax = x[i];
      if ( y[i] > ymax ) ymax = y[i];
    }

    double xrng = xmax-xmin;
    double yrng = ymax-ymin;

    // Use 8 nearest-neighbors (or number of data, n, if n<8):

    if ( n<8 )
    {
      NNN = n;
    }
    else
    {
      NNN = 8;
    }

    // Reasonable defaults for max. allowed distance to first neighbor and
    // maximum search radius:

    Rad1stMax = Math.sqrt((4 * NNN * xrng * yrng) / (Math.PI * n));
    RadSrchMax = Math.sqrt((8 * NNN * xrng * yrng) / (Math.PI * n));

    // Default exponent (inverse-distance-squared interpolation):

    DExpo = 2.0;
  }

  /* ===================================================================== *
   * ---------------------------- SET METHODS ---------------------------- *
   * ===================================================================== */

  /** Method setNnn
   * <p> Sets number of nearest neighbors to use
   * @param n    = The number of nearest neighbors to use
   */

  public void setNnn( int n ) { if ( n > 0 ) NNN = n; }

  /** Method setRad1stMax
   * <p> Sets max. allowed distance to first nearest neighbor
   * @param rad1 = The maximum allowed distance to the first nearest neighbor
   */

  public void setRad1stMax( double rad1 ) { if (rad1 > 0.0) Rad1stMax = rad1; }

  /** Method setRadSrchMax
   * <p> Sets maximum search radius
   * @param rad2 = The maximum search radius
   */

  public void setRadSrchMax( double rad2 ) {if (rad2 > 0.0) RadSrchMax = rad2;}

  /** Method setDexpo
   * <p> Sets distance exponent
   * @param expo = The power of distance to use
   */

  public void setDexpo( double expo ) { if ( expo > 0.0 ) DExpo = expo;	}

  /* ===================================================================== *
   * -------------------------- RETURN METHODS --------------------------- *
   * ===================================================================== */

  /** Method nnn()
   * <p> Returns number of nearest neighbors to use
   * @return NNN = The number of nearest neighbors to use
   */

  public int nnn() { return NNN; }

  /** Method rad1stMax()
   * <p> Returns maximum allowed distance to first nearest neighbor
   * @return Rad1stMax = The maximum allowed distance to the first
   *                     nearest neighbor
   */

  public double rad1stMax() { return Rad1stMax; }

  /** Method radSrchMax()
   * <p> Returns maximum search radius
   * @return RadSrchMax = The maximum search radius
   */

  public double radSrchMax() { return RadSrchMax; }

  /** Method dexpo()
   * <p> Returns distance exponent
   * @return DExpo = The power of distance to use
   */

  public double dexpo() { return DExpo; }
}
