/*
 * @gridPlotAverage.java Version 1.1 07/09/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

//import brine.brineListStruct;       // Brine Data List Structure

/** CLASS gridPlotAverage
 *  <p> This Class will create the Collins Bar Diagram.
 *
 *  @version 1.1 07/09/2015
 *  @author  John R. Victorine
 */

public class gridPlotAverage extends Canvas
{
  private static final int _LINEAR = 0;
  private static final int _LOG    = 1;

  // Other Measured Well Data
  public static final int _FLUID   = fluid.fluidStandardTools._FLUID; // Total Fluid
  public static final int _CUT     = fluid.fluidStandardTools._CUT;   // Oil Cut (%)
  public static final int _BOPD    = fluid.fluidStandardTools._BOPD;  // Barrels of Oil per Day
  public static final int _LEVEL   = fluid.fluidStandardTools._LEVEL; // Fluid Level (ft)
  public static final int _JNTS    = fluid.fluidStandardTools._JNTS;  // Joints in Well
  public static final int _BHP     = fluid.fluidStandardTools._BHP;   // Bottom Hole Pressure
  public static final int _CO2     = fluid.fluidStandardTools._CO2;   // CO2 (%)
  public static final int _MCFD    = fluid.fluidStandardTools._MCFD;  // MCF / D

  private int             iRed        = -1;
  private int             iGreen      = -1;
  private int             iBlue       = -1;

  private double          data[][]    = null;
  private double          dLimits[][] = { {0.0, 1.0, 1.0, 1.0},
                                          {0.0, 1.0, 1.0, 1.0},
                                          {0.0, 1.0, 1.0, 1.0} };
  private double          iGrid[]     = { _LINEAR, _LINEAR, _LINEAR };
  private String          sUnit[]     = { "", "", "" };

  private String          sTitle1     = "Kansas Geological Survey";
  private String          sTitle2     = "Colorlith Brine Data Map";
  private String          sTitle3     = "";
  private String          sTitle4     = "";

  private int iWidth     = 300; // Width of Plot
  private int iHeight    = 650;  // Height of Plot with the Titles
  private int iEWidth    = 300;

  private int iXStart    = 80;   // X Start
  private int imiscXWide = 40;  // Width of Plot Area

  private int iYBegin    = 120; // Starting Point from top for all Plot Areas
  private int iYHigh     = 500; //420; // Height of Plot Areas

  private static final int _TOP  =  20; // Brine Concentration Plot Title
  private static final int _TOP2 =  40; // User Defined Title 1
  private static final int _TOP3 =  60; // User Defined Title 2
  private static final int _TOP4 =  80; // User Defined Title 3

  /** CONSTRUCTOR gridPlotAverage()
   *  <p> This is the constructor for this class.
   *  @param stBrine = Brine Data List Structure
   */

  public gridPlotAverage( int iRed, int iGreen, int iBlue,
                          double data[][], String sTitle4,
                          int iWidth, int iHeight, int iXStart )
  {
	this.iRed    = iRed;
	this.iGreen  = iGreen;
	this.iBlue   = iBlue;

	this.data    = data;
	this.sTitle4 = new String(sTitle4);

    setLimits();

	if (iWidth  > 0) { this.iWidth  = iWidth;  }
	if (iHeight > 0) { this.iHeight = iHeight; }
	if (iXStart > 0) { this.iXStart = iXStart + 80; }
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	data    = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setLimits()
   *  <p> This method will set the limits for the bar and whisker plots
   */

  private void setLimits()
  {
    int    iCount = 0;
    int    iC[]   = null;
    double dMin   = 0.0;
    double dMax   = 0.0;
    double dIncr  = 0.0;

    if (iRed   > -1 ) { iCount++; }
    if (iGreen > -1 ) { iCount++; }
    if (iBlue  > -1 ) { iCount++; }

    iC     = new int[iCount];
    iCount = 0;

    if (iRed > -1)
    {
	  iC[iCount] = iRed;
	  iCount++;
	}

    if (iGreen > -1)
    {
	  iC[iCount] = iGreen;
	  iCount++;
	}

    if (iBlue > -1)
    {
	  iC[iCount] = iBlue;
	  iCount++;
	}

    for (int i=0; i<iCount; i++)
    {
	  if (iC[i] > -1)
	  {
		dMin  = (double) ((int) (data[i][0] * 100.0)) / 100.0;
		dMax  = (double) ((int) (data[i][7] * 100.0)) / 100.0;
		dIncr = (dMax - dMin) / 10.0;

		dLimits[i][0] = dMin;
		dLimits[i][1] = dMax;
		dLimits[i][2] = dIncr;
		dLimits[i][3] =  1;
		iGrid[i]      = _LINEAR;

	    if (iC[i] == _BOPD)
		  sUnit[i] = new String("bopd");
	    else if ((iC[i] == _CUT) || (iC[i] == _CO2))
		  sUnit[i] = new String("%");
	    else if (iC[i] == _FLUID)
		  sUnit[i] = new String("");
	    else if ((iC[i] == _LEVEL) || (iC[i] == _JNTS))
		  sUnit[i] = new String("ft");
	    else if (iC[i] == _MCFD)
		  sUnit[i] = new String("mcf/d");
	    else if (iC[i] == _BHP)
		  sUnit[i] = new String("psig");
      }
	}
  }

  /** Method setLimits()
   *  <p> This method will set the limits for the bar and whisker plots
   *

  private void setLimits()
  {
    int    iCount = 0;
    int    iC[]   = null;

    if (iRed   > -1 ) { iCount++; }
    if (iGreen > -1 ) { iCount++; }
    if (iBlue  > -1 ) { iCount++; }

    iC     = new int[iCount];
    iCount = 0;

    if (iRed > -1)
    {
	  iC[iCount] = iRed;
	  iCount++;
	}

    if (iGreen > -1)
    {
	  iC[iCount] = iGreen;
	  iCount++;
	}

    if (iBlue > -1)
    {
	  iC[iCount] = iBlue;
	  iCount++;
	}

    for (int i=0; i<iCount; i++)
    {
	  if (iC[i] > -1)
	  {
	    if (iC[i] >= _BOPD)
	    {
	  	  dLimits[i][0] = 1;
	 	  dLimits[i][1] = 0.0;
		  dLimits[i][2] = 0.0;
		  dLimits[i][3] = 2;
		  iGrid[i]      = _LOG;
		  sUnit[i]      = new String("bopd");
	    }

	    if ((iC[i] == _CUT) || (iC[i] == _CO2))
	    {
		  dLimits[i][0] =   0.0;
		  dLimits[i][1] = 100.0;
		  dLimits[i][2] =  10.0;
		  dLimits[i][3] =  1;
		  iGrid[i]      = _LINEAR;
		  sUnit[i]      = new String("%");
		}

	    if (iC[i] == _FLUID)
	    {
		  dLimits[i][0] =    0.0;
		  dLimits[i][1] =  500.0;
		  dLimits[i][2] =   50.0;
		  dLimits[i][3] =  1;
		  iGrid[i]      = _LINEAR;
		  sUnit[i]      = new String("");
		}

	    if ((iC[i] == _LEVEL) || (iC[i] == _JNTS))
	    {
		  dLimits[i][0] =    0.0;
		  dLimits[i][1] =  500.0;
		  dLimits[i][2] =   50.0;
		  dLimits[i][3] =  1;
		  iGrid[i]      = _LINEAR;
		  sUnit[i]      = new String("ft");
		}

	    if (iC[i] == _MCFD)
	    {
		  dLimits[i][0] =    0.0;
		  dLimits[i][1] =  200.0;
		  dLimits[i][2] =   20.0;
		  dLimits[i][3] =  1;
		  iGrid[i]      = _LINEAR;
		  sUnit[i]      = new String("mcf/d");
		}

	    if (iC[i] == _BHP)
	    {
	  	  dLimits[i][0] = 1;
	 	  dLimits[i][1] = 0.0;
		  dLimits[i][2] = 0.0;
		  dLimits[i][3] = 5;
		  iGrid[i]      = _LOG;
		  sUnit[i]      = new String("psig");
		}
	  }
	}
  }
*/
  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawTitle()
   *  <p> This method will draw the title Information
   */

  public void drawTitle( Graphics g )
  {
	int         length = 0;
    Font        fsb    = new Font("Monospaced", Font.BOLD, 16);
    FontMetrics fsbm   = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    length = sTitle1.length();
    g.drawString( sTitle1, iXStart + iEWidth/2 - length*4 , _TOP );

    length = sTitle2.length();
    g.drawString( sTitle2, iXStart + iEWidth/2 - length*4 , _TOP2 );

    length = sTitle3.length();
    g.drawString( sTitle3, iXStart + iEWidth/2 - length*4 , _TOP3 );

    length = sTitle4.length();
    g.drawString( sTitle4, iXStart + iEWidth/2 - length*4 , _TOP4 );
  }

  /** Method drawFrame()
   *  <p> This method will draw a Frame around the plot area
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   */

  public void drawFrame( Graphics g,
                         int iXBegin, int iYBegin,
                         int iWide,   int iHigh )
  {
    g.setColor(Color.black);
    g.drawRect(iXBegin, iYBegin, iWide, iHigh);
  }

  /** Method drawXTitle()
   *  <p> This method will draw a X-Axis Title around the plot area
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param sText   - Array of X-Axis Label Text
   *  @param sYUnit  - Y-Axis Unit Text
   */

  public void drawXTitle( Graphics g,
                          int iXBegin,    int iYBegin,
                          int iWide,      int iHigh,
                          String sText[], String sYUnit )
  {
	int iPos = iWide / sText.length;
	int len  = 0;

    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 12) );

    len = sYUnit.length();
    g.drawString(sYUnit, iXBegin-8*len, iYBegin-14);

    g.setFont( new Font("Monospaced", Font.BOLD, 14) );

    for (int i=0; i<sText.length; i++)
    {
	  len = sText[i].length();

      g.drawString(sText[i], iXBegin+(i+1)*iPos-iPos/2-len*4, iYBegin-14);
	}
  }

  /** Method drawXLinearGrid()
   *  <p> This method will draw a X-Axis Linear Grid
   *  @param g           - Graphics pointer.
   *  @param iXBegin     - Starting point from left side of plot
   *  @param iYBegin     - Starting point from top of plot
   *  @param iWide       - Width of Plotting Area
   *  @param iHigh       - Height of Plotting Area
   *  @param dXLower     - X-Axis Start
   *  @param dXUpper     - X-Axis End
   *  @param dXIncrement - X-Axis Increment
   */

  public void drawXLinearGrid( Graphics g,
                               int iXBegin,    int iYBegin,
                               int iWide,      int iHigh,
                               double dXLower, double dXUpper, double dXIncrement)
  {
	int i;
	int iTotal         = 0;
	double ratio       = 0.0;
	int    idraw       = 0;

    // Build the X Linear Grid

    iTotal = (int) Math.ceil((dXUpper-dXLower)/dXIncrement);

    for (i=0; i<iTotal; i++)
    {
      ratio = ((iWide * i * dXIncrement) / (dXUpper-dXLower));
      idraw = iXBegin + (int) ratio;

      if (idraw <= iXBegin+iWide+10)
      {
        g.setColor(Color.lightGray);
        g.drawLine(idraw, iYBegin, idraw, iYBegin+iHigh);
	  }
	}
  }

  /** Method drawYLinearGrid()
   *  <p> This method will draw a Y-Axis Linear Grid
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param dYLower     - Y-Axis Start
   *  @param dYUpper     - Y-Axis End
   *  @param dYIncrement - Y-Axis Increment
   */

  public void drawYLinearGrid( Graphics g,
                               int iXBegin,    int iYBegin,
                               int iWide,      int iHigh,
                               double dYLower, double dYUpper, double dYIncrement)
  {
	int i;
	int iTotal         = 0;
	double ratio       = 0.0;
	int    idraw       = 0;
	double dy          = 0.0;
	String sText       ="";
	int    len         = 0;

//    g.setFont( new Font("Monospaced", Font.BOLD, 10) );
    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    // Build the Y Linear Grid

    iTotal = 1 + (int) Math.ceil((dYUpper-dYLower)/dYIncrement);

    for (i=0; i<iTotal; i++)
    {
      ratio = ((iHigh * i * dYIncrement) / (dYUpper-dYLower));
      idraw = iHigh + iYBegin - (int)ratio;
      g.setColor(Color.lightGray);
      g.drawLine(iXBegin, idraw, iXBegin+iWide, idraw);

      g.setColor(Color.black);
      dy = dYLower + (double) i * dYIncrement;
      dy = Math.round(dy*1000)/1000.0;
//      sText = "" + (int)dy;
      sText = "" + dy;
      len   = sText.length();
      g.drawString(sText, iXBegin-6*len, idraw+4);
	}
  }

  /** Method drawYLogGrid()
   *  <p> This method will draw a Y Axis Log Grid
   *  @param g        - Graphics pointer.
   *  @param iXBegin  - Starting point from left side of plot
   *  @param iYBegin  - Starting point from top of plot
   *  @param iWide    - Width of Plotting Area
   *  @param iHigh    - Height of Plotting Area
   *  @param dYLower  - Y-Axis Start
   *  @param dYCycles - Y-Axis Number of Log Cycles
   */

  public void drawYLogGrid( Graphics g,
                            int iXBegin,    int iYBegin, //int iText,
                            int iWide,      int iHigh,
                            double dYLower, int iYCycle )
  {
	int i,j;
	int    iYinc   = 0;
	int    iMag    = 0;
	double dMag    = 0;
	String sText   = "";
	int    len     = 0;
	double dTemp   = 0.0;
	int    jXinc   = 0;

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    // Build the y Log Grid

    iYinc = iHigh / iYCycle;
    iMag  = (int) dYLower;
    dMag  = dYLower;

    for (i=iYBegin+iHigh; i>iYBegin+iYinc-10; i-=iYinc)
    {
      g.setColor(Color.lightGray);
      g.drawLine(iXBegin, i, iXBegin+iWide, i);

      // Draw the Minor Cycle Grid Lines Light Gray

      for (j=9; j>1; j--)
      {
        dTemp = 1.0 + iYinc * Math.log(j) / Math.log(10.0);
        jXinc = i - (int) dTemp;

        g.setColor(Color.lightGray);
        g.drawLine(iXBegin, jXinc, iXBegin+iWide, jXinc);
      }
    }

    g.setColor(Color.black);
//    g.setFont( new Font("Monospaced", Font.BOLD, 10) );

    sText = "" + (int)dYLower;
    len   = sText.length();

    g.drawString("" + (int)dYLower, iXBegin-len*6, iYBegin+iHigh+4);

    for (i=iYBegin+iHigh; i>iYBegin; i-=iYinc)
    {
      iMag  = iMag*10;
      dMag  = dMag*10;
      sText = "" + dMag; //iMag;
      len   = sText.length();

      if (i-iYinc+4 > iYBegin)
        g.drawString("" + dMag, iXBegin-len*6, i-iYinc+4);
    }
  }

  /** Method drawData()
   *  <p> This method will draw the data
   *  @param g        - Graphics pointer.
   *  @param iXBegin  - Starting point from left side of plot
   *  @param iYBegin  - Starting point from top of plot
   *  @param iWide    - Width of Plotting Area
   *  @param iHigh    - Height of Plotting Area
   *  @param iData    - Brine Data Average Data Identifier
   *  @param iTotal   - Total Number of Brine Data Identifiers
   *  @param iCount   - Count Variable - Start Value
   *  @param dYLower  - Y-Axis Start
   *  @param dYCycles - Y-Axis Number of Log Cycles
   */

  public void drawLogData( Graphics g,
                           int iXBegin,    int iYBegin,
                           int iWide,      int iHigh,
                           int iData,      int iTotal,  int iCount,
                           double dYLower, int iYCycle)
  {
	double dMinimum = Math.log(dYLower) / Math.log(10);
	double dMaximum = dMinimum + iYCycle;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = Math.log(data[iData][brine.brineAvgStruct._MINIMUM]) / Math.log(10);
	      d5   = Math.log(data[iData][brine.brineAvgStruct._5]) / Math.log(10);
	      d25  = Math.log(data[iData][brine.brineAvgStruct._25]) / Math.log(10);
	      d50  = Math.log(data[iData][brine.brineAvgStruct._MEDIAN]) / Math.log(10);
	      dAvg = Math.log(data[iData][brine.brineAvgStruct._MEAN]) / Math.log(10);
	      d75  = Math.log(data[iData][brine.brineAvgStruct._75]) / Math.log(10);
	      d95  = Math.log(data[iData][brine.brineAvgStruct._95]) / Math.log(10);
	      dMax = Math.log(data[iData][brine.brineAvgStruct._MAXIMUM]) / Math.log(10);

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart, i95, 2, i5-i95);

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, iPos-6, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, iPos-6, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, iPos-6, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMin, 8, 2);
          g.fillRect(iStart, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMax, 8, 2);
          g.fillRect(iStart, iMax-3, 2, 8);
		}
	  }
	}
  }

  /** Method drawData()
   *  <p> This method will draw the data
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param iData   - Brine Data Average Data Identifier
   *  @param iTotal  - Total Number of Brine Data Identifiers
   *  @param iCount  - Count Variable - Start Value
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   *

  public void drawData( Graphics g,
                        int iXBegin,    int iYBegin,
                        int iWide,      int iHigh,
                        int iData,      int iTotal,  int iCount,
                        double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = data[iData][brine.brineAvgStruct._MINIMUM];
	      d5   = data[iData][brine.brineAvgStruct._5];
	      d25  = data[iData][brine.brineAvgStruct._25];
	      d50  = data[iData][brine.brineAvgStruct._MEDIAN];
	      dAvg = data[iData][brine.brineAvgStruct._MEAN];
	      d75  = data[iData][brine.brineAvgStruct._75];
	      d95  = data[iData][brine.brineAvgStruct._95];
	      dMax = data[iData][brine.brineAvgStruct._MAXIMUM];

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart, i95, 2, i5-i95);

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, iPos-6, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, iPos-6, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, iPos-6, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMin, 8, 2);
          g.fillRect(iStart, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3, iMax, 8, 2);
          g.fillRect(iStart, iMax-3, 2, 8);
		}
	  }
	}
  }
*/

  /** Method drawData()
   *  <p> This method will draw the data
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param iData   - Brine Data Average Data Identifier
   *  @param iTotal  - Total Number of Brine Data Identifiers
   *  @param iCount  - Count Variable - Start Value
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   */

  public void drawData( Graphics g,
                        int iXBegin,    int iYBegin,
                        int iWide,      int iHigh,
                        int iData,      int iTotal,  int iCount,
                        double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	int    iRGB     = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = data[iData][brine.brineAvgStruct._MINIMUM];
	      d5   = data[iData][brine.brineAvgStruct._5];
	      d25  = data[iData][brine.brineAvgStruct._25];
	      d50  = data[iData][brine.brineAvgStruct._MEDIAN];
	      dAvg = data[iData][brine.brineAvgStruct._MEAN];
	      d75  = data[iData][brine.brineAvgStruct._75];
	      d95  = data[iData][brine.brineAvgStruct._95];
	      dMax = data[iData][brine.brineAvgStruct._MAXIMUM];

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart-(iPos-6)/4, i95, 2, i5-i95);

          // draw color

          for (int k=i95; k<=i5; k++)
          {
			iRGB = 255 - ((255 * ( k - i5 )) / (i95 - i5));
			if (iRGB < 0)   { iRGB = 0; }
			if (iRGB > 255) { iRGB = 255; }

			switch(iData)
			{
			  case 0:
			    g.setColor(new Color(iRGB, 0, 0));
			    break;

			  case 1:
			    g.setColor(new Color(0, iRGB, 0));
			    break;

			  case 2:
			    g.setColor(new Color(0, 0, iRGB));
			    break;
			}
            g.fillRect(iStart+(iPos-6)/4, k-1, (iPos-6)/4, 2);
		  }

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, (iPos-6)/2, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, (iPos-6)/2, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, (iPos-6)/2, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3-(iPos-6)/4, iMin, 8, 2);
          g.fillRect(iStart-(iPos-6)/4, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3-(iPos-6)/4, iMax, 8, 2);
          g.fillRect(iStart-(iPos-6)/4, iMax-3, 2, 8);
		}
	  }
	}
  }

  /** Method drawDataGray()
   *  <p> This method will draw the data
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param iData   - Brine Data Average Data Identifier
   *  @param iTotal  - Total Number of Brine Data Identifiers
   *  @param iCount  - Count Variable - Start Value
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   */

  public void drawDataGray( Graphics g,
                        int iXBegin,    int iYBegin,
                        int iWide,      int iHigh,
                        int iData,      int iTotal,  int iCount,
                        double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 0.0;
	double d5       = 0.0;
	double d25      = 0.0;
	double d50      = 0.0;
	double dAvg     = 0.0;
    double d75      = 0.0;
    double d95      = 0.0;
    double dMax     = 0.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iPos     = iWide / iTotal;
	int    iStart   = 0;

	int    iRGB     = 0;

	if (data != null)
	{
	  if (data[iData] != null)
	  {
		if (data[iData][0] > 0.0)
		{
	      dMin = data[iData][brine.brineAvgStruct._MINIMUM];
	      d5   = data[iData][brine.brineAvgStruct._5];
	      d25  = data[iData][brine.brineAvgStruct._25];
	      d50  = data[iData][brine.brineAvgStruct._MEDIAN];
	      dAvg = data[iData][brine.brineAvgStruct._MEAN];
	      d75  = data[iData][brine.brineAvgStruct._75];
	      d95  = data[iData][brine.brineAvgStruct._95];
	      dMax = data[iData][brine.brineAvgStruct._MAXIMUM];

          dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
          iMax   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
          i95    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
          i75    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
          i50    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
          iAvg   = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
          i25    = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
          i5     = iYBegin + iHigh - (int) dRatio;

          dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
          iMin   = iYBegin + iHigh - (int) dRatio;

          // Small Rectangle from 95% to 5%

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,102,0));
          g.fillRect(iStart-(iPos-6)/4, i95, 2, i5-i95);

          // draw color

          for (int k=i95; k<=i5; k++)
          {
			iRGB = 255 - ((255 * ( k - i5 )) / (i95 - i5));
			if (iRGB < 0)   { iRGB = 0; }
			if (iRGB > 255) { iRGB = 255; }

			switch(iData)
			{
			  case 0:
			    g.setColor(new Color(iRGB, 0, 0));
			    break;

			  case 1:
			    g.setColor(new Color(0, iRGB, 0));
			    break;

			  case 2:
			    g.setColor(new Color(iRGB, iRGB, iRGB));
			    break;
			}
            g.fillRect(iStart+(iPos-6)/4, k-1, (iPos-6)/4, 2);
		  }

          // Large Rectangle from 75% to 25%

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(136,201,155));
          g.fillRect(iStart, i75, (iPos-6)/2, i25-i75);

          // Medium

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,255,255));
          g.fillRect(iStart, i50-1, (iPos-6)/2, 2);

          // Mean

          iStart = iXBegin + iCount * iPos + 3;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart, iAvg-1, (iPos-6)/2, 2);

          // Minimum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3-(iPos-6)/4, iMin, 8, 2);
          g.fillRect(iStart-(iPos-6)/4, iMin-3, 2, 8);

          // Maximum

          iStart = iXBegin + iCount * iPos + iPos/2 - 1;

          g.setColor(new Color(0,0,0));
          g.fillRect(iStart-3-(iPos-6)/4, iMax, 8, 2);
          g.fillRect(iStart-(iPos-6)/4, iMax-3, 2, 8);
		}
	  }
	}
  }

  /** Method drawLegend()
   *  <p> This method will draw the Legend
   *  @param g       - Graphics pointer.
   *  @param iXBegin - Starting point from left side of plot
   *  @param iYBegin - Starting point from top of plot
   *  @param iWide   - Width of Plotting Area
   *  @param iHigh   - Height of Plotting Area
   *  @param dYLower - Y-Axis Start
   *  @param dYUpper - Y-Axis End
   */

  public void drawLegend( Graphics g,
                          int iXBegin,    int iYBegin,
                          int iWide,      int iHigh,
                          double dYLower, double dYUpper )
  {
	double dMinimum = dYLower;
	double dMaximum = dYUpper;

    double dMin     = 1.0;
	double d5       = 2.0;
	double d25      = 3.0;
	double d50      = 4.0;
	double dAvg     = 5.0;
    double d75      = 6.0;
    double d95      = 7.0;
    double dMax     = 8.0;

    double dRatio   = 0.0;

    int    iMin     = 0;
    int    i5       = 0;
    int    i25      = 0;
    int    i50      = 0;
    int    iAvg     = 0;
    int    i75      = 0;
    int    i95      = 0;
    int    iMax     = 0;

	int    iTotal   = 1;
	int    iPos     = iWide / iTotal;
	int    iStart   = 0;
	int    iCount   = 0;

    dRatio = (double) iHigh * (dMax - dMinimum) / (dMaximum - dMinimum);
    iMax   = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d95 - dMinimum) / (dMaximum - dMinimum);
    i95    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d75 - dMinimum) / (dMaximum - dMinimum);
    i75    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d50 - dMinimum) / (dMaximum - dMinimum);
    i50    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (dAvg - dMinimum) / (dMaximum - dMinimum);
    iAvg   = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d25 - dMinimum) / (dMaximum - dMinimum);
    i25    = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (d5 - dMinimum) / (dMaximum - dMinimum);
    i5     = iYBegin + iHigh - (int) dRatio;

    dRatio = (double) iHigh * (dMin - dMinimum) / (dMaximum - dMinimum);
    iMin   = iYBegin + iHigh - (int) dRatio;

    // Small Rectangle from 95% to 5%

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,102,0));
    g.fillRect(iStart, i95, 2, i5-i95);

    g.setColor(new Color(0,0,0));
    g.drawString("Lower 5%", iXBegin+iPos, i5+4);

    g.setColor(new Color(0,0,0));
    g.drawString("Upper 95%", iXBegin+iPos, i95+4);

    // Large Rectangle from 75% to 25%

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(136,201,155));
    g.fillRect(iStart, i75, iPos-6, i25-i75);

    g.setColor(new Color(0,0,0));
    g.drawString("Lower 25%", iXBegin+iPos, i25+4);

    g.setColor(new Color(0,0,0));
    g.drawString("Upper 75%", iXBegin+iPos, i75+4);

    // Medium

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(0,255,255));
    g.fillRect(iStart, i50-1, iPos-6, 2);

    g.setColor(new Color(0,0,0));
    g.drawString("Median", iXBegin+iPos, i50+4);

    // Mean

    iStart = iXBegin + iCount * iPos + 3;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart, iAvg-1, iPos-6, 2);

    g.setColor(new Color(0,0,0));
    g.drawString("Mean", iXBegin+iPos, iAvg+4);

    // Minimum

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart-3, iMin, 8, 2);
    g.fillRect(iStart, iMin-3, 2, 8);

    g.setColor(new Color(0,0,0));
    g.drawString("Minimum", iXBegin+iPos, iMin+4);

    // Maximum

    iStart = iXBegin + iCount * iPos + iPos/2 - 1;

    g.setColor(new Color(0,0,0));
    g.fillRect(iStart-3, iMax, 8, 2);
    g.fillRect(iStart, iMax-3, 2, 8);

    g.drawString("Maximum", iXBegin+iPos, iMax+4);
  }

  /** Method draw()
   * <p> This method will draw the stiff diagram
   *  @param g = Graphics pointer.
   */

  public void draw(Graphics g)
  {
//	int iC[] = { iRed, iGreen, iBlue };
	int    iX          = iXStart;
	int    iLegend     = 0;
    double dXStart     = 0.0; // X-Axis Start
    double dXEnd       = 1.0; // X-Axis End
    double dXIncrement = 1.0; // X-Axis Increment

    double dYStart     = 0.0; // Y-Axis Start
    double dYEnd       = 0.0; // Y-Axis End
    double dYIncrement = 0.0; // Y-Axis Increment
    int    iYCycles    = 0;   // Y-Axis Cycles if plot is log

    String sLbl[]     = { "" };

    int iCount = 0;
    int iC[]   = null;

    if (iRed   > -1 ) { iCount++; }
    if (iGreen > -1 ) { iCount++; }
    if (iBlue  > -1 ) { iCount++; }

    iC     = new int[iCount];
    iCount = 0;

    if (iRed > -1)
    {
	  iC[iCount] = iRed;
	  iCount++;
	}

    if (iGreen > -1)
    {
	  iC[iCount] = iGreen;
	  iCount++;
	}

    if (iBlue > -1)
    {
	  iC[iCount] = iBlue;
	  iCount++;
	}

    drawTitle( g );

    if ((iRed > -1) &&
        (iRed == iGreen) && (iGreen == iBlue))
    {
	  int i = 2;
	  if (iC[i] > -1)
	  {
        dYStart     = dLimits[i][0];
        dYEnd       = dLimits[i][1];
        dYIncrement = dLimits[i][2];
        iYCycles    = (int) dLimits[i][3];

        drawXLinearGrid( g, iX, iYBegin, imiscXWide, iYHigh, dXStart, dXEnd, dXIncrement );
        if (iGrid[i] == _LINEAR)
          drawYLinearGrid( g, iX, iYBegin, imiscXWide, iYHigh, dYStart, dYEnd, dYIncrement );
        else
          drawYLogGrid(    g, iX, iYBegin, imiscXWide, iYHigh, dYStart, iYCycles );
        drawFrame( g, iX, iYBegin, imiscXWide, iYHigh );

        sLbl[0] = new String( brine.brineStandardTools.CURVES[iC[i]][1] );
        drawXTitle( g, iX, iYBegin, imiscXWide, iYHigh, sLbl, sUnit[i] );

        if (iGrid[i] == _LINEAR)
          drawDataGray(    g, iX, iYBegin, imiscXWide, iYHigh, i, 1, 0, dYStart, dYEnd );
        else
          drawLogData( g, iX, iYBegin, imiscXWide, iYHigh, i, 1, 0, dYStart, iYCycles );

        iX      = iX + 100;
        iLegend = iX - 40;
      }
	}
	else
	{
//    for (int i=0; i<3; i++)
      for (int i=0; i<iCount; i++)
      {
	    if (iC[i] > -1)
	    {
          dYStart     = dLimits[i][0];
          dYEnd       = dLimits[i][1];
          dYIncrement = dLimits[i][2];
          iYCycles    = (int) dLimits[i][3];


          drawXLinearGrid( g, iX, iYBegin, imiscXWide, iYHigh, dXStart, dXEnd, dXIncrement );
          if (iGrid[i] == _LINEAR)
            drawYLinearGrid( g, iX, iYBegin, imiscXWide, iYHigh, dYStart, dYEnd, dYIncrement );
          else
            drawYLogGrid(    g, iX, iYBegin, imiscXWide, iYHigh, dYStart, iYCycles );
          drawFrame( g, iX, iYBegin, imiscXWide, iYHigh );

          sLbl[0] = new String( fluid.fluidStandardTools.CURVES[iC[i]][1] );
          drawXTitle( g, iX, iYBegin, imiscXWide, iYHigh, sLbl, sUnit[i] );

          if (iGrid[i] == _LINEAR)
            drawData(    g, iX, iYBegin, imiscXWide, iYHigh, i, 1, 0, dYStart, dYEnd );
          else
            drawLogData( g, iX, iYBegin, imiscXWide, iYHigh, i, 1, 0, dYStart, iYCycles );

          iX      = iX + 100;
          iLegend = iX - 40;
	    }
  	  }
    }

    dYStart     = 0.0;
    dYEnd       = 14.0;

    drawLegend( g, iLegend, iYBegin, imiscXWide, iYHigh, dYStart, dYEnd );
  }

  /** Method paint( Graphics g )
   * <p> This method will paint the stiff diagram Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
    draw(g);
  }
}

/*
 *  @version 1.1 07/09/2015
 *  @author  John Victorine
 */
